package struclust;

import java.util.LinkedList;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import struclust.eval.EvalResult;
import struclust.eval.testsetups.PublicationADMA2017TS;

/**
 * Class to run the evaluation
 * 
 * @author Till Schäfer
 */
public class RunEval {
    private static final Logger logger = LoggerFactory.getLogger(RunEval.class);
    private static final String eol = System.getProperty("line.separator").toString();

    /**
     * Main method: run evaluation
     * 
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        logger.info("Starting RunEval");
        Thread.setDefaultUncaughtExceptionHandler(new UncaughtExceptionLogger());
        
        LinkedList<EvalResult> results = PublicationADMA2017TS.ReproduceAnchorQuery.run();

        for (EvalResult result : results) {
            logger.debug("{}{}", eol, result.toString());
        }
    }
    
    private static class UncaughtExceptionLogger implements Thread.UncaughtExceptionHandler {
        @Override
        public void uncaughtException(Thread t, Throwable e) {
            logger.error("Uncaught Expeption in RunEval in thread " + t.getName() + " (" + t.toString() + ")", e);
        }
    }
}
