package struclust.eval.comparison;

import graph.Graph;

import java.util.Collection;

import struclust.Cluster;
import struclust.Describable;

/**
 * Comparison measure for two clusterings
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public interface ClusteringComparison<NL, EL, G extends Graph<NL, EL>> extends Describable {
    /**
     * Calculate the measure between two clusterings
     * 
     * Cluster and graph must not be null and the cluster representatives must
     * not be empty.
     * 
     * @param clusters1
     *            the first clustering
     * @param clusters2
     *            the second clustering. if the validation is not symmetric,
     *            this argument should be the ground truth.
     * @return the comparison value between clustering1 and clustering2
     */
    public double calc(Collection<Cluster<NL, EL, G>> clusters1, Collection<Cluster<NL, EL, G>> clusters2);

    /**
     * @return the maximal value of this comparison measure or NAN if there is
     *         no such value
     */
    public double maxValue();

}
