package struclust.eval.comparison;

import graph.Graph;

import java.util.Collection;

import struclust.Cluster;
import struclust.eval.util.InformationTheoreticClusteringMeasures;

/**
 * Normalised variation of information. (NMI_joint)
 * 
 * see: Nguyen Xuan Vinh, Julien Epps, and James Bailey. 2010. Information
 * Theoretic Measures for Clusterings Comparison: Variants, Properties,
 * Normalization and Correction for Chance. J. Mach. Learn. Res. 9999 (December
 * 2010), 2837-2854.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class NormVariationOfInformationCC<NL, EL, G extends Graph<NL, EL>> implements ClusteringComparison<NL, EL, G> {

    @Override
    public String getDescription() {
        return "NormVariationOfInformationCC";
    }

    @Override
    public double calc(Collection<Cluster<NL, EL, G>> clusters1, Collection<Cluster<NL, EL, G>> clusters2) {
        ContingencyMatrix<NL, EL, G> matrix = new ContingencyMatrix<>(clusters1, clusters2);
        double value = InformationTheoreticClusteringMeasures.mutualInformation(matrix)
                / InformationTheoreticClusteringMeasures.jointEntropy(matrix);
        
        assert !Double.isNaN(value);
        return value;
    }

    @Override
    public double maxValue() {
        return 1;
    }

}
