package struclust.eval.modules.clustering;

import graph.Graph;

import java.util.Collections;
import java.util.function.Function;
import java.util.stream.Stream;

import struclust.Cluster;
import struclust.StructuralClustering;
import struclust.eval.EvalResult;

/**
 * Generates one {@link EvalResult} containing statistics about the cluster
 * sizes (minimum, maximum, average, median) and another EvalResult with all
 * cluster sizes.
 * 
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class ClustSizeEM<NL, EL, G extends Graph<NL, EL>> extends StatisticsAndValuesEM<NL, EL, G, Cluster<NL, EL, G>> {

    /**
     * Constructor.
     * 
     * @param clust
     *            The configured clustering algorithm
     */
    public ClustSizeEM(StructuralClustering<NL, EL, G> clust) {
        super(clust, "ClusterSize");
    }

    @Override
    protected Function<? super Cluster<NL, EL, G>, ? extends Stream<? extends Cluster<NL, EL, G>>> mapper() {
        return x -> Collections.singleton(x).stream();
    }

    @Override
    protected double getValue(Cluster<NL, EL, G> c) {
        return c.size();
    }

}
