package struclust.eval.modules.clustering;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import graph.Graph;
import struclust.Cluster;
import struclust.StructuralClustering;
import struclust.eval.EvalException;
import struclust.eval.EvalResult;
import struclust.eval.comparison.ClusteringComparison;

/**
 * Compares the calculated clustering to another clustering (the ground truth)
 * based on specified {@link ClusteringComparison}.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class ClusteringComparisonEM<NL, EL, G extends Graph<NL, EL>> extends PrecompClustEM<NL, EL, G> {

    private ClusteringComparison<NL, EL, G> measure;
    private List<Cluster<NL, EL, G>> groundTruth;

    /**
     * Constructor
     * 
     * @param clust
     *            the configured clustering
     * @param measure
     *            the measure for clustering comparison
     * @param groundTruth
     *            the clustering for comparison considered as ground truth
     */
    public ClusteringComparisonEM(StructuralClustering<NL, EL, G> clust, ClusteringComparison<NL, EL, G> measure,
            List<Cluster<NL, EL, G>> groundTruth) {
        super(clust);
        this.measure = measure;
        this.groundTruth = groundTruth;
    }

    @Override
    protected Collection<EvalResult> runInternal(List<Cluster<NL, EL, G>> clustering) throws EvalException {
        EvalResult retVal = new EvalResult("ClusteringComparison_" + measure.getClass().getName());

        retVal.addMetadata(clust);
        retVal.addMetadata("ClusteringComparisonEM measure", measure.getDescription());

        retVal.addResult("measurement_" + measure.getDescription(), measure.calc(clustering, groundTruth));

        return Collections.singleton(retVal);
    }

}
