package struclust.eval.modules.clustering;

import graph.Graph;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Stream;

import struclust.Cluster;
import struclust.SharedMemorySC;
import struclust.comparison.CoverageCGD;
import struclust.eval.EvalException;
import struclust.eval.EvalResult;
import struclust.graph.GraphContainer;

/**
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class CoverageEM<NL, EL, G extends Graph<NL, EL>> extends
        StatisticsAndValuesEM<NL, EL, G, GraphContainer<NL, EL, G>> {
    private CoverageCGD<NL, EL, G> dist;
    private HashMap<GraphContainer<NL, EL, G>, Cluster<NL, EL, G>> gcToCluster = new HashMap<>();

    /**
     * Constructor
     * 
     * @param clust
     *            the configured clustering
     */
    public CoverageEM(SharedMemorySC<NL, EL, G> clust) {
        super(clust, "AverageGraphCoverage");
        dist = new CoverageCGD<>(clust.getConf());
    }

    @Override
    public Collection<EvalResult> run(List<Cluster<NL, EL, G>> testData) throws EvalException {
        synchronized (gcToCluster) {
            testData.stream().forEach(c -> c.stream().forEach(gc -> gcToCluster.put(gc, c)));
            Collection<EvalResult> results = super.run(testData);
            return results;
        }
    }

    @Override
    protected Function<? super Cluster<NL, EL, G>, ? extends Stream<? extends GraphContainer<NL, EL, G>>> mapper() {
        return x -> x.stream();
    }

    @Override
    protected double getValue(GraphContainer<NL, EL, G> gc) {
        return 1 - dist.calc(gcToCluster.get(gc), gc);
    }

}
