package struclust.eval.modules.clustering;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.commons.io.FilenameUtils;

import graph.Graph;
import struclust.Cluster;
import struclust.SharedMemorySC;
import struclust.eval.EvalException;
import struclust.eval.EvalResult;
import struclust.eval.metamodules.FileSaverEMM;

/**
 * Saves the injection results. This is useful, when some kind of repeating meta
 * module is used and one wants to save the injection results after each run.
 * 
 * Results will be stored to folderpath/injectionEM/
 * 
 * @author Till Schäfers
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class InjectionSaverEM<NL, EL, G extends Graph<NL, EL>> extends PrecompClustEM<NL, EL, G> {

    private String folderPath;

    /**
     * Constructor
     * 
     * @param clust
     *            the configured clustering algorithm
     * @param folderPath
     *            resutls will be stored to this path
     */
    public InjectionSaverEM(SharedMemorySC<NL, EL, G> clust, String folderPath) {
        super(clust);
        this.folderPath = folderPath;
    }

    @Override
    protected Collection<EvalResult> runInternal(List<Cluster<NL, EL, G>> clustering) throws EvalException {
        FileSaverEMM injectionsaver = new FileSaverEMM(FilenameUtils.concat(folderPath, "injectionEM/injection.txt"),
                true, true);
        injectionsaver.process(((SharedMemorySC<NL, EL, G>) clust).getInjectionResults());
        return Collections.emptyList();
    }

}
