package struclust.eval.modules.clustering;

import graph.Graph;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import struclust.Cluster;
import struclust.StructuralClustering;
import struclust.eval.EvalException;
import struclust.eval.EvalResult;

import com.google.common.base.Preconditions;

/**
 * Write out avgRepToMemberSize^powers / numClusters.
 * 
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class RTMSToNumClustersEM<NL, EL, G extends Graph<NL, EL>> extends PrecompClustEM<NL, EL, G> {

    private double[] powers;

    /**
     * Constructor
     * 
     * @param clust
     *            The configured clustering algorithm
     * @param powers
     *            the powers of this array are applied to avgRepToMemberSize
     *            before the value is divided by numClusters
     */
    public RTMSToNumClustersEM(StructuralClustering<NL, EL, G> clust, double[] powers) {
        super(clust);
        Preconditions.checkNotNull(powers);
        this.powers = powers;
    }

    @Override
    protected Collection<EvalResult> runInternal(List<Cluster<NL, EL, G>> clustering) throws EvalException {
        EvalResult retVal = new EvalResult("RTMSToNumClusters");
        retVal.addMetadata(clust.getConf());

        double avgRTMS = clustering.stream().mapToDouble(c -> c.getRepToMemberSize() * c.size()).sum();
        avgRTMS /= clustering.stream().mapToInt(c -> c.size()).sum();

        for (double power : powers) {
            retVal.addResult("avgRTMSToNumClusters_" + Double.toString(power),
                    Math.pow(avgRTMS, power) / testData.size());
        }

        return Collections.singleton(retVal);
    }
}
