package struclust.eval.modules.clustering;

import graph.Graph;
import graph.GraphSize;

import java.util.function.Function;
import java.util.stream.Stream;

import struclust.Cluster;
import struclust.SharedMemorySC;
import struclust.eval.EvalResult;
import struclust.graph.GraphContainer;

/**
 * Generates one {@link EvalResult} containing statistics about the cluster
 * representatives sizes (in terms of nodes and edges) (minimum, maximum,
 * average, median) and another EvalResult with all representative sizes.
 * 
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class RepSizeEM<NL, EL, G extends Graph<NL, EL>> extends
        StatisticsAndValuesEM<NL, EL, G, GraphContainer<NL, EL, G>> {

    private GraphSize s;

    /**
     * Constructor.
     * 
     * @param clust
     *            The configured clustering algorithm
     */
    public RepSizeEM(SharedMemorySC<NL, EL, G> clust) {
        super(clust, "RepresentativeSize");
        s = clust.getConf().gSize;
    }

    @Override
    protected Function<? super Cluster<NL, EL, G>, ? extends Stream<? extends GraphContainer<NL, EL, G>>> mapper() {
        return x -> x.getRepresentatives().stream();
    }

    @Override
    protected double getValue(GraphContainer<NL, EL, G> t) {
        return t.size(s);
    }

}
