package struclust.eval.modules.clustering;

import graph.Graph;

import java.util.Collections;
import java.util.function.Function;
import java.util.stream.Stream;

import struclust.Cluster;
import struclust.StructuralClustering;
import struclust.eval.EvalResult;

/**
 * Generates one {@link EvalResult} containing statistics about the cluster
 * RepToMemberSizes (minimum, maximum, average, median) and another EvalResult
 * with all RepToMemberSizes
 * 
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class RepToMemberSizeEM<NL, EL, G extends Graph<NL, EL>> extends
        StatisticsAndValuesEM<NL, EL, G, Cluster<NL, EL, G>> {

    /**
     * Constructor
     * 
     * @param clust
     *            The configured clustering algorithm
     */
    public RepToMemberSizeEM(StructuralClustering<NL, EL, G> clust) {
        super(clust, "RepToMemberSize");
    }

    @Override
    protected double getValue(Cluster<NL, EL, G> c) {
        return c.getRepToMemberSize();
    }

    @Override
    protected Function<? super Cluster<NL, EL, G>, ? extends Stream<? extends Cluster<NL, EL, G>>> mapper() {
        return x -> Collections.singleton(x).stream();
    }

}
