package struclust.eval.modules.clustering;

import graph.Graph;
import graph.GraphSize;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import struclust.Cluster;
import struclust.ClusterFactory;
import struclust.eval.EvalException;
import struclust.eval.EvalResult;
import struclust.eval.modules.AbstractEM;
import struclust.graph.GraphContainer;
import struclust.graph.Graphs;
import struclust.util.Clusters;

/**
 * This is actually a fake module, that does not run any new tests, but uses a
 * pre-defined clustering and stores it as test data. Together with the series
 * module, this allows to apply other evaluation modules on the stored test
 * data.
 * 
 * Please note, that many modules assume object level equality for the graphs in
 * the test data. Be careful to compare graphs read from files with graphs read
 * from an other file. Use
 * {@link Graphs#getIsomorphicGraphMapping(Collection, Collection)} to avoid
 * such situations.
 *
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class UsePredefinedClusteringEM<NL, EL, G extends Graph<NL, EL>> extends AbstractEM<List<Cluster<NL, EL, G>>> {

    private GraphSize gSize;
    List<? extends List<GraphContainer<NL, EL, G>>> clustering;

    /**
     * Constructor
     * 
     * @param clustering
     *            the clustering structure.
     * @param gSize
     *            the {@link GraphSize} for the clusters
     */
    public UsePredefinedClusteringEM(List<? extends List<GraphContainer<NL, EL, G>>> clustering, GraphSize gSize) {
        this.clustering = clustering;
        this.gSize = gSize;
    }

    @Override
    public Collection<EvalResult> run() throws EvalException {
        testData = Clusters.toClusterListGC(clustering, new ClusterFactory<>(gSize));
        return Collections.emptyList();
    }

    @Override
    public Collection<EvalResult> run(List<Cluster<NL, EL, G>> testData) throws EvalException {
        throw new UnsupportedOperationException(
                "UsePredefinedClusteringEM cannot run a test with precomputed test data");
    }

    @Override
    public boolean canUsePrecomputedTestData() {
        return false;
    }

}
