package struclust.eval.modules.comparison;

import graph.Graph;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.commons.math3.random.EmpiricalDistribution;
import org.apache.commons.math3.stat.descriptive.SummaryStatistics;

import struclust.comparison.GraphGraphDistance;
import struclust.eval.EvalException;
import struclust.eval.EvalResult;
import struclust.graph.GraphContainer;

/**
 * Calculates the distance matrix and generates a histogram of the values.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class GGDHistogramEM<NL, EL, G extends Graph<NL, EL>> extends PrecompGGDMatrixEM<NL, EL, G> {

    private int binCount;

    /**
     * Constructor
     * 
     * @param dist
     *            distance measure to measure
     * @param graphs
     *            graphs to test the distance measure on
     * @param binCount
     *            the number of bins for the Histogram
     */
    public GGDHistogramEM(GraphGraphDistance<NL, EL, G> dist, List<GraphContainer<NL, EL, G>> graphs, int binCount) {
        super(dist, graphs);
        this.binCount = binCount;
    }

    @Override
    protected Collection<EvalResult> runInternal(double[][] testData) throws EvalException {
        EvalResult result = new EvalResult("GGDHistogram");
        result.addMetadata("histogram graph graph distance", dist);
        result.addMetadata("histogram bin count", binCount);

        double[] values = values(testData);
        EmpiricalDistribution distrib = new EmpiricalDistribution(binCount);
        distrib.load(values);

        double[] upperBounds = distrib.getUpperBounds();
        List<SummaryStatistics> binStats = distrib.getBinStats();

        for (int i = 0; i < upperBounds.length; i++) {
            result.addResult(Double.toString(upperBounds[i]), binStats.get(i).getN());
        }

        return Collections.singleton(result);
    }

}
