package struclust.eval.modules.comparison;

import graph.Graph;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.commons.math3.random.EmpiricalDistribution;
import org.apache.commons.math3.stat.descriptive.StatisticalSummary;

import struclust.comparison.GraphGraphDistance;
import struclust.eval.EvalException;
import struclust.eval.EvalResult;
import struclust.graph.GraphContainer;

/**
 * Calculates the distance matrix and generates some statistics of the values.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class GGDStatisticsEM<NL, EL, G extends Graph<NL, EL>> extends PrecompGGDMatrixEM<NL, EL, G> {

    /**
     * Constructor
     * 
     * @param dist
     *            distance measure to measure
     * @param graphs
     *            graphs to test the distance measure on
     */
    public GGDStatisticsEM(GraphGraphDistance<NL, EL, G> dist, List<GraphContainer<NL, EL, G>> graphs) {
        super(dist, graphs);
    }

    @Override
    protected Collection<EvalResult> runInternal(double[][] testData) throws EvalException {
        EvalResult result = new EvalResult("GGDStatistics");
        result.addMetadata("statistics graph graph distance", dist);

        double[] values = values(testData);
        EmpiricalDistribution distrib = new EmpiricalDistribution();
        distrib.load(values);

        StatisticalSummary sampleStats = distrib.getSampleStats();
        result.addResult("min", sampleStats.getMin());
        result.addResult("max", sampleStats.getMax());
        result.addResult("mean", sampleStats.getMean());
        result.addResult("stdev", sampleStats.getStandardDeviation());
        result.addResult("median", distrib.inverseCumulativeProbability(0.5));
        result.addResult("0.1quantile", distrib.inverseCumulativeProbability(0.1));
        result.addResult("0.25quantile", distrib.inverseCumulativeProbability(0.25));
        result.addResult("0.75quantile", distrib.inverseCumulativeProbability(0.75));
        result.addResult("0.9quantile", distrib.inverseCumulativeProbability(0.9));

        return Collections.singleton(result);
    }

}
