package struclust.eval.modules.dataset;

import graph.Graph;
import graph.GraphSize;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.math3.random.EmpiricalDistribution;
import org.apache.commons.math3.stat.descriptive.SummaryStatistics;

import struclust.eval.EvalException;
import struclust.eval.EvalResult;
import struclust.eval.modules.AbstractEM;

/**
 * Creates a histogram of graph sizes
 * 
 * @author Till Schäfer
 */
public class GraphSizeHistogramEM extends AbstractEM<Object> {

    private int binCount;
    private Collection<? extends Graph<?, ?>> graphs;

    /**
     * Constructor
     * 
     * @param graphs
     *            the graphs to compute the histogram for
     * @param binCount
     *            the number of bins for the histogram
     */
    public GraphSizeHistogramEM(Collection<? extends Graph<?, ?>> graphs, int binCount) {
        this.graphs = graphs;
        this.binCount = binCount;
    }

    @Override
    public Collection<EvalResult> run() throws EvalException {
        Collection<EvalResult> results = new ArrayList<>(GraphSize.values().length);

        for (GraphSize gs : GraphSize.values()) {
            EvalResult result = new EvalResult("GraphSizeHistogramEM");
            result.addMetadata("histogram bin count", binCount);
            result.addMetadata("graph size measure", gs.toString());

            double[] values = values(gs);
            EmpiricalDistribution distrib = new EmpiricalDistribution(binCount);
            distrib.load(values);

            double[] upperBounds = distrib.getUpperBounds();
            List<SummaryStatistics> binStats = distrib.getBinStats();

            for (int i = 0; i < upperBounds.length; i++) {
                result.addResult(Double.toString(upperBounds[i]), binStats.get(i).getN());
            }
            results.add(result);
        }

        return results;
    }

    private double[] values(GraphSize gs) {
        double values[] = new double[graphs.size()];

        int i = 0;
        for (Graph<?, ?> g : graphs) {
            values[i] = g.size(gs);
            i++;
        }

        return values;
    }

    @Override
    public Collection<EvalResult> run(Object testData) throws EvalException {
        throw new UnsupportedOperationException("GraphSizeHistogramEM cannot run a test with precomputed test data");
    }

    @Override
    public boolean canUsePrecomputedTestData() {
        return false;
    }

}
