package hsahn.algorithm.graph.coloring;

import hsahn.graph.Graph;
import hsahn.graph.Graph.Vertex;
import hsahn.graph.properties.VertexArray;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

/**
 * An approximative greedy algorithm for graph coloring.
 */
public class GreedyGraphColorer extends GraphColorer {

	public int color(Graph g, VertexArray<Integer> c) {
		int maxClassId = Integer.MIN_VALUE;
		
		ArrayList<Vertex> vertices = new ArrayList<Vertex>();
		for (Vertex v : g.vertices()) {
			vertices.add(v);
		}
		Collections.sort(vertices, new Comparator<Vertex>() {
			@Override
            public int compare(Vertex u, Vertex v) {
				int du = u.getDegree();
				int dv = v.getDegree();
				if (du > dv) return -1;
				if (dv > du) return 1;
				return 0;
			}
		});
		
		for (Vertex v : vertices) {
//		for (Vertex v : g.vertices()) {
			int classId = -1;
			boolean valid;
			do {
				classId++;
				valid = true;
				for (Vertex w : v.neighbors()) {
					Integer wClassId = c.get(w);
					if (wClassId == null) continue;
					if (wClassId == classId) {
						valid = false;
						break;
					}
				}
			} while (!valid);
			c.set(v, classId);
			if (classId > maxClassId) {
				maxClassId = classId;
			}
		}
		return maxClassId + 1;
	}
	
	/**
	 * Implements the NUMBER-SORT procedure described in:
	 * 
	 * Tomita, Kameda; An efficient branch-and-bound algorithm 
	 * for finding a maximum clique with computational experiments;
	 * J. Glob. Optim. (2007) 37:95-111
	 *  
	 *  This sorting depends on the greedy approach and might be
	 *  less effective with other GraphColorers.
	 */
	public ArrayList<Vertex> numberSort(Graph g) {
		ArrayList<Vertex> r = new ArrayList<Vertex>(g.getVertexCount());
		
		ArrayList<ArrayList<Vertex>> ccp = colorClassPartition(g);
		for (ArrayList<Vertex> cc : ccp) {
			r.addAll(cc);
		}
		
		return r;
	}
}
