package hsahn.algorithm.graph.isomorphism.labelrefinement;

import hsahn.graph.Graph;
import hsahn.graph.Graph.Edge;
import hsahn.graph.Graph.Vertex;
import hsahn.graph.LGraph;
import hsahn.graph.properties.EdgeArray;
import hsahn.graph.properties.VertexArray;
import hsahn.graph.properties.VertexProperty;

public class BridgeRefiner extends LabelRefiner<LGraph<?,?>, LGraph<String,String>>{
	
	@Override
    public LGraph<String, String> refineGraph(LGraph<?, ?> lg) {
		Graph g = lg.getGraph();
		VertexArray<?> va = lg.getVertexLabel();
		EdgeArray<?> ea = lg.getEdgeLabel();
		
		VertexArray<String> vaL = new VertexArray<String>(g);
		EdgeArray<String> eaL = new EdgeArray<String>(g);
		
		for (Vertex v : g.vertices()) {
			vaL.set(v, va.get(v).toString());
		}
		for (Edge e : g.edges()) {
			eaL.set(e, ea.get(e).toString());
		}
		
		for (Edge e : g.edges()) {
			VertexProperty<Integer> found = new VertexArray<Integer>(g);
			if (!findPath(e.getFirstVertex(), e.getFirstVertex(), e.getSecondVertex(), found))
				eaL.set(e, eaL.get(e)+" b");
		}
		
		return new LGraph<String, String>(g, vaL, eaL);
	}
	
	
	private static boolean findPath(Vertex v, Vertex s, Vertex t, VertexProperty<Integer> found) {
		if (v==t) return true;
		found.set(v, 1);
		for (Vertex w : v.neighbors()) {
			if (found.get(w)==null && ( v!=s || w!=t )) {
				if (findPath(w, s, t, found))
					return true;
			}
		}
		return false;
	}

}
