package hsahn.algorithm.graph.vectorization;

import hsahn.concepts.Transformation;
import hsahn.datastructure.SparseFeatureVector;
import hsahn.datastructure.Triple;
import hsahn.graph.Graph;
import hsahn.graph.Graph.Edge;
import hsahn.graph.LGraph;
import hsahn.graph.properties.EdgeArray;
import hsahn.graph.properties.VertexArray;

/**
 * Computes a feature vector for a given graph counting the occurences of
 * extended edge labels, i.e. a triple (l(u), l((u,v)), l(v)) for an edge 
 * (u, v), where l assigns a label to the two endpoints and the edge itself,
 * respectively. Note that for each edge two such triples are generated
 * where the position of u and v is swapped.
 * 
 * @author kriege
 *
 * @param <V> vertex label type
 * @param <E> edge label type
 */
public class EdgeVectorizer<V,E> implements Transformation<LGraph<V,E>, SparseFeatureVector<Triple<V,E,V>>> {

	@Override
    public SparseFeatureVector<Triple<V, E, V>> transform(LGraph<V, E> in) {
		Graph g = in.getGraph();
		VertexArray<V> va = in.getVertexLabel();
		EdgeArray<E> ea = in.getEdgeLabel();
		
		SparseFeatureVector<Triple<V, E, V>> r = new SparseFeatureVector<Triple<V, E, V>>();
		for (Edge e : g.edges()) {
			V lu = va.get(e.getFirstVertex());
			V lv = va.get(e.getSecondVertex());
			E le = ea.get(e);
			r.increaseCount(new Triple<V, E, V>(lu, le, lv));
			r.increaseCount(new Triple<V, E, V>(lv, le, lu));
		}
		
		return r;
	}

}
