package hsahn.algorithm.shortestpath;

import hsahn.datastructure.Matrix;
import hsahn.graph.AdjMatrixGraph;
import hsahn.graph.Digraph;
import hsahn.graph.Graph;
import hsahn.graph.Graph.Edge;
import hsahn.graph.Graph.Vertex;
import hsahn.graph.LGraph;
import hsahn.graph.properties.EdgeArray;

public class SPTools {
	private static AllPairShortestPath apspModule = new FloydWarshallAPSP();
	
	/**
	 * Note: vertex labels will not be cloned; the object is the same as stored in lg!
	 * @param g
	 * @return result from type AdjMatrixGraph (high memory consumption, but some operations are faster!
	 */
	public static <V> LGraph<V,Integer> getAPSPTransformation(LGraph<V, ?> lg) {
		Graph g = lg.getGraph();
		if (g instanceof Digraph) {
			throw new UnsupportedOperationException("This opertion is currently not supported for directed graphs.");
		} else {
			Matrix<Integer> dist = apspModule.computeShortestPaths(g);
			int n = g.getVertexCount();
			AdjMatrixGraph r = new AdjMatrixGraph(n, n*(n-1));
			EdgeArray<Integer> rl = new EdgeArray<Integer>(r);
			for (int i=0; i<g.getVertexCount(); i++) {
				Vertex v = r.createVertex();
				assert(v.getIndex() == i);
			}
			for (int i=0; i<n; i++) {
				for (int j=i+1; j<n; j++) {
					Edge e = r.createEdge(r.getVertex(i), r.getVertex(j));
					rl.set(e, dist.get(i, j));
				}
			}
			
			return new LGraph<V, Integer>(r, lg.getVertexLabel(), rl);
		}
	}

}
