package hsahn.comparison.distance;

import hsahn.comparison.kernel.Kernel;

/**
 * Computes the Euclidean distance in the reproducing kernel
 * hilbert space.
 * 
 * @author kriege
 *
 * @param <T>
 */
public class KernelDistance<T> implements Distance<T> {

	private Kernel<T> kernel;
	
	public KernelDistance(Kernel<T> kernel) {
		this.kernel = kernel;
	}

	@Override
	public double compute(T g1, T g2) {
		double self1 = kernel.compute(g1, g1);
		double self2 = kernel.compute(g2, g2);
		double value = kernel.compute(g1, g2);
		return Math.sqrt(self1+self2-2*value);
	}

}
