package hsahn.comparison.distance;

import hsahn.comparison.kernel.Kernel;

/**
 * Uses the 1 - normalized kernel value as dissimilarity.
 * 
 * The kernel values itself is normalized to [0,1] by
 * 
 * <pre>
 * k'(x,y) = k(x,y) / sqrt(k(x,x) * k(y,y))
 * </pre>
 * 
 * @author Till Schäfer
 *
 * @param <T>
 */
public class NormalizedKernelDistance<T> implements Distance<T> {

    private Kernel<T> kernel;

    /**
     * Constructor
     * 
     * @param kernel
     *            the used {@link Kernel}
     */
    public NormalizedKernelDistance(Kernel<T> kernel) {
        this.kernel = kernel;
    }

    @Override
    public double compute(T g1, T g2) {
        double self1 = kernel.compute(g1, g1);
        double self2 = kernel.compute(g2, g2);
        double value = kernel.compute(g1, g2);
        return 1 - (value / Math.sqrt(self1 * self2));
    }

}
