package hsahn.comparison.kernel;

/**
 * Returns the result of several kernel functions, computed by a single 
 * call of the method {@link #computeComposition(Object, Object)}. Each 
 * of the returned values, i.e. the elements at the same index, must 
 * yield a positive semidefinite kernel matrix.
 * 
 * @author kriege
 *
 * @param <T> type of the objects that can be compared
 * by this implementation
 */
public abstract class CompositeKernel<T> implements Kernel<T> {
	
	/**
	 * {@inheritDoc}
	 * Computes a single kernel value by adding up all kernel
	 * function values.
	 */
	public double compute(T g1, T g2) {
		double r = 0;
		double[] comp = computeComposition(g1, g2);
		for (double c : comp) {
			r += c;
		}
		
		return r;
	}

	/**
	 * Computes the kernel values for the given objects.
	 * @param g1 first object under comparison
	 * @param g2 second object under comparison
	 * @return the result of the kernel functions computed
	 * by this composite kernel
	 */
	public abstract double[] computeComposition(T g1, T g2);
	
	/**
	 * @return the number of kernel values computed
	 */
	public abstract int getComponentCount();
	
}
