package hsahn.comparison.kernel.graph;

import hsahn.comparison.kernel.Kernel;
import hsahn.graph.Graph;
import hsahn.graph.Graph.Edge;
import hsahn.graph.Graph.Vertex;
import hsahn.graph.LGraph;
import hsahn.graph.properties.EdgeArray;
import hsahn.graph.properties.VertexArray;

/**
 * @inheritDocs
 * The direct product graphs connects vertex pairs with common adjacency only.
 * 
 * Deprecated convention: Directly computes vertex and edge weight array.
 */
@Deprecated
public class WeightedDirectProductGraph<V,E> extends WeightedProductGraph<V, E> {
	
	/**
	 * Creates a new instance that can be used to subsequently create multiple
	 * direct product graphs. 
	 * @param edgeKernel kernel used to compare edges
	 * @param vertexKernel kernel used to compare vertices
	 */
	public WeightedDirectProductGraph(Kernel<? super E> edgeKernel, Kernel<? super V> vertexKernel) {
		super(edgeKernel, vertexKernel);
	}
	
	/**
	 * @inheritDocs
	 */
	public void compute(LGraph<V,E> lg1, LGraph<V,E> lg2) {
		Graph g1 = lg1.getGraph();
		Graph g2 = lg2.getGraph();
		VertexArray<V> va1 = lg1.getVertexLabel();
		VertexArray<V> va2 = lg2.getVertexLabel();
		EdgeArray<E> ea1 = lg1.getEdgeLabel();
		EdgeArray<E> ea2 = lg2.getEdgeLabel();
		
		compG1.clear();
		compG2.clear();
		
		vertexWeights = initVertexValue(g1.getVertexCount()*g2.getVertexCount(), false);
		// compute compatible vertices
		for (Vertex v1 : g1.vertices()) {
			for (Vertex v2 : g2.vertices()) {
				double value = vKernel.compute(va1.get(v1), va2.get(v2));
				if (value != 0) {
					vertexWeights[compG1.size()] = value;
					compG1.add(v1);
					compG2.add(v2);
				}
			}
		}

		n = compG1.size();
		m = 0;
		
		edgeWeights = initEdgeValue(n, false);
		for (int i=0; i<n; i++) {
			Vertex u1 = compG1.get(i);
			Vertex u2 = compG2.get(i);
			for (int j=i+1; j<n; j++) {
				Vertex v1 = compG1.get(j);
				Vertex v2 = compG2.get(j);
				double v = 0;
				Edge e1 = g1.getEdge(u1, v1);
				Edge e2 = g2.getEdge(u2, v2);
				if (e1 != null && e2 != null) {
					v = eKernel.compute(ea1.get(e1), ea2.get(e2));
				}
				edgeWeights[i][j] = edgeWeights[j][i] = v;
				if (v != 0) m++;
			}
		}
	}
	
	

}
