package hsahn.comparison.kernel.graph;

import hsahn.comparison.kernel.Kernel;
import hsahn.comparison.kernel.graph.ProductGraph.PVertex;
import hsahn.graph.Graph;
import hsahn.graph.Graph.Edge;
import hsahn.graph.Graph.Vertex;
import hsahn.graph.LGraph;
import hsahn.graph.properties.EdgeArray;
import hsahn.graph.properties.VertexArray;

/**
 * @inheritDocs
 * The direct product graphs connects vertex pairs with common adjacency only. 
 */
public class WeightedDirectProductGraphBuilderDense<V,E> extends ProductGraphBuilder<V, E, ProductGraph> {
	
	/**
	 * @inheritDocs
	 */
	public WeightedDirectProductGraphBuilderDense(Kernel<? super E> edgeKernel, Kernel<? super V> vertexKernel) {
		super(edgeKernel, vertexKernel);
	}

	/**
	 * @inheritDocs
	 */
	@Override
    public ProductGraph build(LGraph<V,E> lg1, LGraph<V,E> lg2) {
		Graph g1 = lg1.getGraph();
		Graph g2 = lg2.getGraph();
		VertexArray<V> va1 = lg1.getVertexLabel();
		VertexArray<V> va2 = lg2.getVertexLabel();
		EdgeArray<E> ea1 = lg1.getEdgeLabel();
		EdgeArray<E> ea2 = lg2.getEdgeLabel();
		
		ProductGraph pg = new ProductGraph(g1, g2);
		// compute compatible vertices
		for (Vertex v1 : g1.vertices()) {
			V l1 = va1.get(v1);
			for (Vertex v2 : g2.vertices()) {
				V l2 = va2.get(v2);
				double value = vertexKernel.compute(l1, l2);
				if (value != 0) {
					pg.createVertex(v1, v2, value);
				}
			}
		}

		int n = pg.getVertexCount();
		
		for (int i=0; i<n; i++) {
			PVertex pu = pg.getVertex(i);
			Vertex u1 = pu.getFirst();
			Vertex u2 = pu.getSecond();
			for (int j=i+1; j<n; j++) {
				PVertex pv = pg.getVertex(j);
				Vertex v1 = pv.getFirst();
				Vertex v2 = pv.getSecond();
				if (u1 == v1 || u2 == v2) continue;
				Edge e1 = g1.getEdge(u1, v1);
				Edge e2 = g2.getEdge(u2, v2);
				if (e1 != null && e2 != null) {
					double value = edgeKernel.compute(ea1.get(e1), ea2.get(e2));
					if (value != 0) {
						pg.createEdge(pu, pv, value);
					}
				}
			}
		}
			
		return pg;
	}

}
