package hsahn.graph;

import java.util.ArrayList;

public class AdjListDiGraph extends AbstractExtendibleGraph implements Digraph {
	
	ArrayList<AdjListDiVertex> vertices;
	ArrayList<AdjListDiEdge> edges;
	
	public AdjListDiGraph(int n, int m) {
		vertices = new ArrayList<AdjListDiVertex>(n);
		edges = new ArrayList<AdjListDiEdge>(m);
	}
	
	public AdjListDiGraph() {
		vertices = new ArrayList<AdjListDiVertex>();
		edges = new ArrayList<AdjListDiEdge>();
	}

	public Iterable<AdjListDiEdge> edges() {
		return edges;
	}

	public AdjListDiEdge getEdge(int index) {
		return edges.get(index);
	}

	public int getVertexCount() {
		return vertices.size();
	}
	
	public int getEdgeCount() {
		return edges.size();
	}
	
	public AdjListDiEdge getEdge(Vertex u, Vertex v) {
		return (AdjListDiEdge)ImplementationHelper.getEdge(u, v);
	}

	public boolean hasEdge(Vertex u, Vertex v) {
		return getEdge(u, v) != null;
	}

	public AdjListDiVertex getVertex(int index) {
		return vertices.get(index);
	}

	public AdjListDiEdge getDiEdge(DiVertex u, DiVertex v) {
		for (DiEdge e : u.outEdges()) 
			if (e.getTargetVertex() == v)
				return (AdjListDiEdge)e;
		return null;
	}
	
	public boolean hasDiEdge(DiVertex u, DiVertex v) {
		return getEdge(u, v) != null;
	}

	public Iterable<AdjListDiVertex> vertices() {
		return vertices;
	}
	
	public AdjListDiVertex createVertex() {
		AdjListDiVertex v = new AdjListDiVertex(vertices.size());
		vertices.add(v);
		
		notifyVertexCreated(v);
		return v;
	}
	
	public AdjListDiEdge createEdge(Vertex u, Vertex v) {
		AdjListDiEdge e = new AdjListDiEdge((AdjListDiVertex)u, (AdjListDiVertex)v, edges.size());
		edges.add(e);
		
		notifyEdgeCreated(e);
		return e;
	}
	
	
	public class AdjListDiVertex implements DiVertex {
		
		int index;
		ArrayList<AdjListDiEdge> edges;
		ArrayList<AdjListDiEdge> inEdges;
		ArrayList<AdjListDiEdge> outEdges;
		
		public AdjListDiVertex(int index) {
			this.index = index;
			edges = new ArrayList<AdjListDiEdge>();
			inEdges = new ArrayList<AdjListDiEdge>();
			outEdges = new ArrayList<AdjListDiEdge>();
		}

		public int inDegree() {
			return inEdges.size();
		}

		public Iterable<AdjListDiEdge> inEdges() {
			return inEdges;
		}

		public Iterable<? extends AdjListDiVertex> inNeighbours() {
			return ImplementationHelper.createInNeighborIterator(this);
		}

		public int outDegree() {
			return outEdges.size();
		}

		public Iterable<? extends DiEdge> outEdges() {
			return outEdges;
		}

		public Iterable<? extends DiVertex> outNeighbours() {
			return ImplementationHelper.createOutNeighborIterator(this);
		}

		public Iterable<AdjListDiEdge> edges() {
			return edges;
		}

		public int getDegree() {
			return edges.size();
		}

		public Iterable<? extends Vertex> neighbors() {
			return ImplementationHelper.createNeighborIterator(this);
		}

		public int getIndex() {
			return index;
		}
		
	}
	
	public class AdjListDiEdge implements DiEdge {
		
		int index;
		AdjListDiVertex source, target;

		public AdjListDiEdge(AdjListDiVertex source, AdjListDiVertex target, int index) {
			this.index = index;
			this.source = source;
			this.target = target;
		}

		public DiVertex getSourceVertex() {
			return source;
		}

		public DiVertex getTargetVertex() {
			return target;
		}

		public Vertex getFirstVertex() {
			return source;
		}

		public Vertex getOppositeVertex(Vertex v) {
			return (source == v) ? target : source;
		}

		public Vertex getSecondVertex() {
			return target;
		}

		public int getIndex() {
			return index;
		}
		
	}

}
