package hsahn.mcs;

import hsahn.datastructure.Pair;
import hsahn.graph.AdjMatrixGraph;
import hsahn.graph.Graph;
import hsahn.graph.LGraph;
import hsahn.graph.properties.EdgeArray;
import hsahn.graph.properties.VertexArray;

public class AssociationGraph extends AdjMatrixGraph {

    Graph g1;
    Graph g2;
    VertexArray<?> va1;
    VertexArray<?> va2;
    EdgeArray<?> ea1;
    EdgeArray<?> ea2;

    VertexArray<Pair<Vertex, Vertex>> map;
    EdgeArray<Boolean> cEdge;

    protected AssociationGraph() {
    }

    public AssociationGraph(Graph g1, Graph g2) {
        this.g1 = g1;
        this.g2 = g2;
        build();
    }

    public AssociationGraph(LGraph<?, ?> lg1, LGraph<?, ?> lg2) {
        this.g1 = lg1.getGraph();
        this.g2 = lg2.getGraph();
        this.va1 = lg1.getVertexLabel();
        this.va2 = lg2.getVertexLabel();
        this.ea1 = lg1.getEdgeLabel();
        this.ea2 = lg2.getEdgeLabel();
        build();
    }

    public Pair<Vertex, Vertex> getFactors(Vertex v) {
        return map.get(v);
    }

    protected void build() {

        map = new VertexArray<Pair<Vertex, Vertex>>(this, g1.getVertexCount() * g2.getVertexCount());
        cEdge = new EdgeArray<Boolean>(this);

        for (Vertex v1 : g1.vertices()) {
            for (Vertex v2 : g2.vertices()) {
                if (isCompatible(v1, v2)) {
                    Vertex v = createVertex();
                    map.set(v, new Pair<Vertex, Vertex>(v1, v2));
                }
            }
        }

        for (int i = 0; i < getVertexCount(); i++) {
            Vertex u = getVertex(i);
            Pair<Vertex, Vertex> pu = map.get(u);
            Vertex u1 = pu.getFirst();
            Vertex u2 = pu.getSecond();
            for (int j = i + 1; j < getVertexCount(); j++) {
                Vertex v = getVertex(j);
                Pair<Vertex, Vertex> pv = map.get(v);
                Vertex v1 = pv.getFirst();
                Vertex v2 = pv.getSecond();
                if (u1 == v1 || u2 == v2)
                    continue;
                Edge e1 = g1.getEdge(u1, v1);
                Edge e2 = g2.getEdge(u2, v2);
                if (!isCompatible(e1, e2))
                    continue;
                Edge e = createEdge(u, v);
                cEdge.set(e, e1 != null);
            }
        }
    }

    protected boolean isCompatible(Vertex e1, Vertex e2) {
        if (va1 == null || va2 == null)
            return true;
        if (e1 == e2)
            return true; // both may be null
        Object v1 = va1.get(e1);
        Object v2 = va2.get(e2);
        if (v1 == v2)
            return true;
        return v1.equals(v2);
    }

    /**
     * Returns true iff both edges are null or the same object or have an equal
     * label or both no label.
     * 
     * @param e1
     * @param e2
     * @return
     */
    protected boolean isCompatible(Edge e1, Edge e2) {
        if (e1 == null ^ e2 == null)
            return false;
        if (e1 == e2)
            return true; // both may be null
        if (ea1 == null || ea2 == null)
            return true;
        Object v1 = ea1.get(e1);
        Object v2 = ea2.get(e2);
        if (v1 == v2)
            return true;
        return v1.equals(v2);
    }

    public Graph getG1() {
        return g1;
    }

    public Graph getG2() {
        return g2;
    }

    public boolean isCEdge(Edge e) {
        return cEdge.get(e);
    }

}