package hsahn.mcs;

import hsahn.graph.Graph;
import hsahn.graph.Graph.Edge;
import hsahn.graph.Graph.Vertex;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.Set;

public class KCliqueFinder {

	private Graph graph;
	private int k;
	private int states;
	private int kCliqueCount;
	private CliqueHander ch;
	private ArrayList<Edge>[] adj;
	
	public KCliqueFinder(Graph graph, int k) {
		this.graph = graph;
		this.k = k;
		this.adj = new ArrayList[graph.getVertexCount()];
	}
	
	
	public int computeKCliques() {
		states = 0;
		kCliqueCount = 0;
		
		ArrayList<Vertex> cand = new ArrayList<Vertex>(graph.getVertexCount());
		for (int i=0; i< graph.getVertexCount(); i++) {
			Vertex v = graph.getVertex(i);
			cand.add(v);
			adj[i] = new ArrayList<Edge>(v.getDegree());
			for (Edge e : v.edges())
				adj[i].add(e);
			Collections.sort(adj[i], new CKCliqueFinder.EdgeComparator(v));
		}
		
		maxClique(0, cand);
		
		return kCliqueCount;
	}
	
	private void maxClique(int size, ArrayList<Vertex> cand) {
		states++;

		while (!cand.isEmpty() /*&& (size + cand.size() >= k)*/) {

			Vertex v = popCandidate(cand);
			
			//kCliqueCount++;
			kCliqueCount += size+1;
			if (size+1 == k) {
				continue;
			}
			
			ArrayList<Vertex> newCand = newIntersectionSet(cand, v);
			maxClique(size+1, newCand);
		}
	}
	
	private Vertex popCandidate(ArrayList<Vertex> cand) {
		return cand.remove(cand.size()-1);
	}
	
	private ArrayList<Vertex> newIntersectionSet(ArrayList<Vertex> setP, Vertex x) {
		ArrayList<Vertex> result = new ArrayList<Vertex>();
		Iterator<Vertex> itP = setP.iterator();
		Vertex v = null;
		Vertex vP = itP.hasNext() ? itP.next() : null;
		for (Edge e : adj[x.getIndex()]) {
			v = e.getOppositeVertex(x);
			
			while (itP.hasNext() && vP.getIndex() < v.getIndex()) {
				vP = itP.next();
			}
			if (v == vP) {
				result.add(v);
				continue;
			}
		}
		
		return result;
	}
	
	public int getStates() {
		return states;
	}
	
	public static interface CliqueHander {
		public void handleClique(Set<Vertex> clique);
	}
}
