package hsahn.mcs;

import hsahn.algorithm.graph.coloring.GreedyGraphColorer;
import hsahn.graph.Graph;
import hsahn.graph.Graph.Vertex;
import hsahn.graph.properties.VertexArray;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

public class MaximumCliqueFinder {

	private int maxSize;
	private Graph graph;
	private int states;
	private VertexArray<Integer> colors;
	
	public MaximumCliqueFinder(Graph graph) {
		this.graph = graph;
	}
	
	
	public int computeMaxSize() {
		maxSize = 0;
		states = 0;
		
		List<Vertex> cand = new ArrayList<Vertex>(graph.getVertexCount());
//		for (Vertex v : graph.vertices())
//			cand.add(v);
		GreedyGraphColorer ggc = new GreedyGraphColorer();
		ArrayList<ArrayList<Vertex>> ccp = ggc.colorClassPartition(graph);
		//TODO avoid one colorer call!
		colors = ggc.color(graph);
		
		for (int i=ccp.size()-1; i>=0; i--) {
//		for (ArrayList<Vertex> c : ggc.colorClassPartition(graph)) {
//			cand.addAll(c);
			Collections.sort(ccp.get(i), new Comparator<Vertex>() {
				@Override
                public int compare(Vertex u, Vertex v) {
					int du = u.getDegree();
					int dv = v.getDegree();
					if (du > dv) return 1;
					if (dv > du) return -1;
					return 0;
				}
			});
			cand.addAll(ccp.get(i));
		}
		
//		for (Vertex v : cand) System.out.print(colors.get(v)+" "); System.out.println();
		
		maxClique(0, cand);
		
		return maxSize;
	}
	
	private void maxClique(int size, List<Vertex> cand) {
		states++;
		if (cand.isEmpty()) {
			if (size > maxSize)
				maxSize = size;
		} else {
//			OrbitPartition op = MaxSymBreaking.PARTIAL_AUTOMORPHISM.createOrbitPartition(graph, cand);
//			OrbitPartition op = MaxSymBreaking.DUMMY.createOrbitPartition(graph, cand);
			while (size + cand.size() > maxSize) {
				Vertex v = nextCandidate(cand);
				
//				//sanity check
//				int max = Integer.MIN_VALUE;
//				for (Vertex vert : cand) max = Math.max(max, colors.get(vert));
//				System.out.println(colors.get(v)+" "+max);
//				for (Vertex vert : cand) System.out.print(colors.get(vert)+" "); System.out.println();
//				// check ascending order
//				int c = colors.get(v);
//				for (Vertex vert : cand) {
//					if (c < colors.get(vert)) {
//						System.out.println("EROOROR");
//						System.exit(-1);
//					}
//					c = colors.get(vert);
//				}
				
				if (size + colors.get(v) + 1 > maxSize) {
					List<Vertex> newCand = newIntersectionSet(cand, v/*.neighbors()*/);
					maxClique(size+1, newCand);
				}
				cand.remove(v);
//				cand.removeAll(op.getOrbit(v));
//				if (op.getOrbit(v).size() > 1)
//					System.out.println(op.getOrbit(v).size());
			}
		}
	}
	
	private Vertex nextCandidate(List<Vertex> cand) {
		Iterator<Vertex> it = cand.iterator();
		Vertex v = it.next();
		it.remove();
		return v;
	}
	
	private List<Vertex> newIntersectionSet(List<Vertex> set1, Vertex u/*Iterable<? extends Vertex> set2*/) {
		// TODO try TreeSet and BitSet
//		Set<Vertex> result = new TreeSet<Vertex>(new Comparator<Vertex>() {
//			public int compare(Vertex o1, Vertex o2) {
//				int i1 = o1.getIndex();
//				int i2 = o2.getIndex();
//				return i1<i2 ? -1 : (i1==i2 ? 0 : 1);
//			}
//		});

		List<Vertex> result = new ArrayList<Vertex>(Math.min(set1.size(), u.getDegree()));
		for (Vertex v : set1)
			if (graph.hasEdge(u,v))
				result.add(v);

		
		// this is more efficient, but does not preserve ordering!
//		Set<Vertex> result = new LinkedHashSet<Vertex>();
//		for (Vertex v : set2)
//			if (set1.contains(v))
//				result.add(v);
		
//		Set<Vertex> cand = new LinkedHashSet<Vertex>();
//		InducedSubgraph g2 = new InducedSubgraph(graph, result);
//		GreedyGraphColorer ggc = new GreedyGraphColorer();
//		for (ArrayList<Vertex> c : ggc.colorClassPartition(g2)) {
//			cand.addAll(c);
//		}
//		
//		return cand;
		
		return result;
	}
	
	public int getStates() {
		return states;
	}
	
}
