package hsahn.mcs;

import hsahn.datastructure.Pair;
import hsahn.graph.Graph;
import hsahn.graph.Graph.Edge;
import hsahn.graph.Graph.Vertex;

import java.util.LinkedList;

public class PartialMapping {

    public static long stateCount = 0;
    public static long pmCount = 0;

    public static final boolean DEBUG = false;

    protected Graph g1;
    protected Graph g2;

    protected int[] mapPH;
    protected int[] mapHP;

    protected int size;

    public PartialMapping(Graph g1, Graph g2) {
        pmCount++;
        this.g1 = g1;
        this.g2 = g2;

        mapPH = new int[g1.getVertexCount()];
        mapHP = new int[g2.getVertexCount()];
        for (int i = 0; i < mapPH.length; i++)
            mapPH[i] = -1;
        for (int i = 0; i < mapHP.length; i++)
            mapHP[i] = -1;

        size = 0;
    }

    public boolean isComplete() {
        return (size == g1.getVertexCount());
    }

    public Vertex mapToHostNode(Vertex u) {
        return g2.getVertex(mapPH[u.getIndex()]);
    }

    public Vertex mapToPatternNode(Vertex v) {
        return g1.getVertex(mapHP[v.getIndex()]);
    }

    public LinkedList<Pair<Vertex, Vertex>> getMapping() {
        LinkedList<Pair<Vertex, Vertex>> mapping = new LinkedList<Pair<Vertex, Vertex>>();

        for (int i = 0; i < mapPH.length; i++)
            if (mapPH[i] != -1)
                mapping.add(new Pair<Vertex, Vertex>(g1.getVertex(i), g2.getVertex(mapPH[i])));

        return mapping;
    }

    public int getSize() {
        return size;
    }

    public void extend(Vertex u, Vertex v) {
        stateCount++;
        if (DEBUG)
            System.out.println("added (" + u.getIndex() + ", " + v.getIndex() + ")");

        mapPH[u.getIndex()] = v.getIndex();
        mapHP[v.getIndex()] = u.getIndex();

        size++;
    }

    public void remove(Vertex u, Vertex v) {
        if (DEBUG)
            System.out.println("removed (" + u.getIndex() + ", " + v.getIndex() + ")");

        mapPH[u.getIndex()] = -1;
        mapHP[v.getIndex()] = -1;

        size--;
    }

    /**
     * 
     * @param u
     * @param v
     * @param uEdges
     *            all edges incident to u and v with v already mapped
     * @return
     */
    public boolean isFeasiblePair(Vertex u, Vertex v, Iterable<? extends Edge> uEdges, Iterable<? extends Edge> vEdges) {
        // u or v already mapped
        if (mapHP[v.getIndex()] != -1 || mapPH[u.getIndex()] != -1) {
            if (DEBUG)
                System.out.println("Not feasible: Node already in mapping");

            return false;
        }

        // check existence of g1 edges in g2
        for (Edge e : uEdges) {
            Vertex u2 = e.getOppositeVertex(u);
            int iu2Mapped = mapPH[u2.getIndex()];
            if (iu2Mapped != -1) {
                Edge mappedEdge = g2.getEdge(g2.getVertex(iu2Mapped), v);

                // false iff host graph does not contain the edge or only a
                // wrong labeled edge
                if (mappedEdge == null) {
                    if (DEBUG)
                        System.out.println("Not feasible: Missing edge");

                    return false;
                }
            }
        }

        // check existence of g2 edges in g1
        for (Edge e : vEdges) {
            Vertex v2 = e.getOppositeVertex(v);
            int iv2Mapped = mapHP[v2.getIndex()];
            if (iv2Mapped != -1) {
                Edge mappedEdge = g1.getEdge(g1.getVertex(iv2Mapped), u);

                // false iff host graph does not contain the edge or only a
                // wrong labeled edge
                if (mappedEdge == null) {
                    if (DEBUG)
                        System.out.println("Not feasible: Missing edge");

                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public String toString() {
        StringBuffer s = new StringBuffer();

        for (int i = 0; i < mapPH.length; i++)
            s.append(i + " -> " + mapPH[i] + "\n");

        return s.toString();
    }

}
