package index.features.extractor;

import graph.Edge;
import graph.Graph;
import graph.Node;
import index.util.LabelCounter;

/**
 * Counts the labels in a given distance from each node.
 */
public class ConnectivityExtractor extends FeatureExtractor<LabelCounter[]> {

    private int maxDepth;
    private LabelCounter[] labelCounter;

    public ConnectivityExtractor(Graph graph, int maxDepth) {
        this.graph = graph;
        this.maxDepth = maxDepth;
        this.labelCounter = new LabelCounter[maxDepth + 1];
        for (int i = 0; i < labelCounter.length; i++) {
            labelCounter[i] = new LabelCounter(128);
        }
    }

    public void extractFeatures() {
        // boolean[] visited = new boolean[graph.getNodeCount()];
        for (Node n : graph.nodes()) {
            // pathSearch(n, 0, visited);
            pathSearch(null, n, 0);
        }
    }

    private void pathSearch(Node parent, Node<? extends Object, ? extends Object> u, int depth) {

        labelCounter[depth].add(getLabel(u));

        for (Edge e : u.getEdges()) {
            Node v = e.getOppositeNode(u);
            if (parent != v && depth < maxDepth) {
                pathSearch(u, v, depth + 1);
            }
        }
    }

    /*
     * private void pathSearch(Node u, int depth, boolean[] visited) {
     * 
     * visited[u.getIndex()] = true; labelCounter[depth].add(getLabel(u));
     * 
     * for (Edge e : u.getEdges()) { Node v = e.getOppositeNode(u); if
     * (!visited[v.getIndex()] && depth < maxDepth) {
     * 
     * pathSearch(v, depth+1, visited); } } visited[u.getIndex()] = false; }
     */

    public LabelCounter[] getResult() {
        return labelCounter;
    }
}
