/*
 * ScaffoldHunter
 * Copyright (C) 2006-2008 Karsten Klein, Stefan Wetzel, Arbia Ben Ahmed,
 * Anke Arndt, Philipp Bdenbender, Vanessa Bembenek, Adalbert Gorecki,
 * Nils Kriege, Sergej Rakov, Michael Rex, Gebhard Schrader, Henning Wagner,
 * Andr Wiesniewski, Cengizhan Ycel
 *
 * This file is part of ScaffoldHunter.
 *
 * ScaffoldHunter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ScaffoldHunter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package index.util;

import java.sql.*;
import java.util.Properties;

public class IndexSQLController {

    public static final boolean DEBUG = false;
    public static final String DEFAULT_DRIVER_NAME = "com.mysql.jdbc.Driver";
    
    private Connection dbConn;
    
    public IndexSQLController(Connection dbConn) {
    	this.dbConn = dbConn;
    } 
    
    public IndexSQLController(String url, String user, String pwd) throws ClassNotFoundException, SQLException {
    	Properties connectionInfo = new Properties();
    	connectionInfo.setProperty("user", user);
    	connectionInfo.setProperty("password", pwd);
    	connectionInfo.setProperty("autoReconnect", "true");
    	
    	Class.forName(DEFAULT_DRIVER_NAME);
    	
    	dbConn = DriverManager.getConnection(url, connectionInfo);
    } 
   
    /**
     * Closes the connection to the database
     * 
     * @throws SQLException
     */
    public void close() throws SQLException{
    	if (dbConn != null) {
    		dbConn.close();
    	}
    }
    
    public Connection getConnection() {
    	return dbConn;
    }
    

    /**
     * Searches for database-entries matching the query given by strSQL 
     * 
     * @param sqlQuery encloses the query
     * @return ResultSet contains the result(s) given back my the database in a table-like style
     * @throws SQLException
     */
    public ResultSet searchInDB (String sqlQuery) throws SQLException {
   		return query(sqlQuery);
    } 
    
    private ResultSet query(String sqlQuery) throws SQLException {
    	long m=0;
    	if (DEBUG) {
    		System.out.print(sqlQuery+"...");
    		System.out.flush();
    		m = System.currentTimeMillis();
    	}
    	
    	Statement statement = dbConn.createStatement();
    	ResultSet rs = statement.executeQuery(sqlQuery);
 
    	if (DEBUG)
    		System.out.println("done ["+ (System.currentTimeMillis()-m) +" msec]");
    	
    	return rs;
    }
    
    public ResultSet queryStreamResult(String sqlQuery) throws SQLException {
    	long m=0;
    	if (DEBUG) {
    		System.out.print(sqlQuery+"...");
    		System.out.flush();
    		m = System.currentTimeMillis();
    	}
    	
    	Statement statement = dbConn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
    	// Allows fetching the results row-by-row with MySQL.
    	// The implementation may depend on the database driver!
    	statement.setFetchSize(Integer.MIN_VALUE);
    	ResultSet rs = statement.executeQuery(sqlQuery);
 
    	if (DEBUG)
    		System.out.println("first chunk ["+ (System.currentTimeMillis()-m) +" msec]");
    	
    	return rs;
    }
    
    /**
     * writes a query into the database
     * 
     * @param sqlQuery encloses the query
     * @throws SQLException
     */
    public void writeInDB(String sqlQuery) throws SQLException{
   		Statement stm = dbConn.createStatement();
   		stm.executeUpdate(sqlQuery);
   		stm.close();
    }
}
