package index.util;

import java.io.Serializable;
import java.util.BitSet;
import java.util.Collection;
import java.util.LinkedList;

/**
 * Allows fast union/intersect operations for sets of integers
 * from a small basic set by using fast BitSets operations
 */
public class IntSet implements Serializable { //, Collection<Integer> {
	
	private static final long serialVersionUID = 1;
	
	BitSet b;
	
	public IntSet() {
		b = new BitSet();
	}
	
	/**
	 * Copy constructor.
	 */
	public IntSet(IntSet s) {
		this.b = (BitSet)s.b.clone();
	}
	
	public IntSet(Collection<Integer> c) {
		this();
		this.addAll(c);
	}
	
	public void add(Integer i) {
		b.set(i);
	}
	
	public void remove(Integer i) {
		b.clear(i);
	}
	
	public void addAll(Collection<Integer> c) {
		if (c != null)
			for (Integer i : c)
				add(i);
	}
	
	public void removeAll(Collection<Integer> c) {
		for (Integer i : c)
			remove(i);
	}
	
	public void union(IntSet set) {
		b.or(set.b);
	}
	
	public void intersect(IntSet set) {
		b.and(set.b);
	}
	
	public LinkedList<Integer> getValues() {
		LinkedList<Integer> res = new LinkedList<Integer>();
		for (int i = b.nextSetBit(0); i >= 0; i = b.nextSetBit(i+1)) {
			res.add(i);
		}
		return res;
	}
	
}
