package match;

import graph.Graph;
import graph.Node;
import match.pattern.SearchPlan.Extension;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Matcher that includes the pattern symmetry breaking conditions presented by Grochow et. al. in 2007
 * @author Markus Kloss
 *
 * @param <NL> The {@link Node} label
 * @param <EL> The {@link Edge} label
 */
public class MatcherVF2Auto<NL, EL> extends MatcherVF2 
{
    private static final Logger logger = LoggerFactory.getLogger(MatcherVF2Auto.class);
    
    protected PartialMappingAuto pma;

    /**
     * Constructor
     * @param patternGraph The pattern {@link Graph}
     * @param hostGraph The host {@link Graph}
     */
    public MatcherVF2Auto(Graph patternGraph, Graph hostGraph) 
    {
        super(patternGraph, hostGraph);
        
        this.pma = new PartialMappingAuto(this.searchPattern, hostGraph);
    }
    
    @Override
    public boolean match() {
        
        if (pma.isComplete())
            return handleMatch(pma);
        
        Node patternCandidate = nextPatternCandidate();
        Node hostCandidate = pma.nextTerminalHostNode(patternCandidate, null);
        while (hostCandidate != null) {
            
            if (DEBUG)
                System.out.println("Current Candidate: " + patternCandidate.getIndex() + " -> "
                        + hostCandidate.getIndex());
            
            if (pma.isFeasibleCandidate(patternCandidate, hostCandidate)) {
                pma.extend(patternCandidate, hostCandidate);
                if (match())
                    return true;
                pma.remove(patternCandidate, hostCandidate);
            }
            
            hostCandidate = pma.nextTerminalHostNode(patternCandidate, hostCandidate);
        }
        
        return false;
    }
    
    private Node nextPatternCandidate() {
        Extension ext = searchPlan.getExtension(pma.getSize());
        return ext.getNode();
    }
    

}
