package struclust.comparison;

import graph.Graph;
import struclust.Cluster;
import struclust.Describable;

/**
 * Distance between two {@link Cluster}s
 * 
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public interface ClusterClusterDistance<NL, EL, G extends Graph<NL, EL>> extends Describable {
    /**
     * Calculate the distance between two {@link Cluster}s
     * 
     * Both {@link Cluster}s must not be null and the cluster representatives
     * must not be empty.
     * 
     * @param cluster1
     *            the first {@link Cluster}
     * @param cluster2
     *            the second {@link Cluster}
     * @return the distance between the {@link Cluster}s
     */
    public double calc(Cluster<NL, EL, G> cluster1, Cluster<NL, EL, G> cluster2);

    /**
     * @return if clusters with empty representatives are supported
     */
    public boolean supportsEmptyRepresentatives();

    /**
     * @return if empty clusters are supported
     */
    public boolean supportsEmptyClusters();

}
