package struclust.comparison;

import graph.Graph;
import hsahn.comparison.distance.NormalizedKernelDistance;
import hsahn.comparison.kernel.Kernel;
import hsahn.graph.GraphTools;
import hsahn.graph.LGraph;
import struclust.graph.GraphContainer;

/**
 * Wrapper class to use {@link NormalizedKernelDistance} in struclust
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class NormKernelGGD<NL, EL, G extends Graph<NL, EL>> implements GraphGraphDistance<NL, EL, G> {

    private NormalizedKernelDistance<LGraph<NL, EL>> dist;
    private Kernel<LGraph<NL, EL>> kernel;

    /**
     * Constructor
     * 
     * @param kernel
     *            the used kernel for distance calculation
     */
    public NormKernelGGD(Kernel<LGraph<NL, EL>> kernel) {
        this.kernel = kernel;
        dist = new NormalizedKernelDistance<>(kernel);
    }

    @Override
    public double calc(GraphContainer<NL, EL, G> graph1, GraphContainer<NL, EL, G> graph2) {
        /*
         * PERF: This can be done faster by precomputing the explicit mapping
         * with ExplicitMappingKernel#computeExplicitMapping() in advance and
         * apply these mappings to the InnerProductKernel afterwards. This would
         * also save the Graph conversion.
         * 
         * => This will need a redesign of precomputed values (i.e.
         * fingerprints, explicit mappings, etc). modularization!
         */
        LGraph<NL, EL> lg1 = GraphTools.convert(graph1);
        LGraph<NL, EL> lg2 = GraphTools.convert(graph2);
        return dist.compute(lg1, lg2);
    }

    @Override
    public double maxDist() {
        /*
         * XXX: the meaning actually depends on the used kernel
         * 
         * For FLRW kernel this might be quite usable, as it is possible to
         * specify the path length and this corresponds to the common subgraph
         * size. I.e. a path is only found if there exist a common subgraph with
         * that size.
         * 
         * For WS-SP oder WS-ST this might be unusable on the other hand, as a
         * single common node would mean that maximum distance is not reached.
         */
        return 1;
    }

    @Override
    public String getDescription() {
        return "NormKernelGGD: " + kernel.getID();
    }

}
