package struclust.comparison;

import graph.Graph;
import match.MatcherFast;
import struclust.graph.GraphContainer;

/**
 * The value of this distance is 1 if there exists an embedding of the pattern
 * graph in the target/host graph and 0 if not. The distance is non-symmetric.
 * 
 * This is a variant of {@link SubIsoGGD}, which uses a Fingerprint
 * prefiltering.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class SubIsoFpGGD<NL, EL, G extends Graph<NL, EL>> implements GraphGraphDistance<NL, EL, G> {

    @Override
    public double calc(GraphContainer<NL, EL, G> pattern, GraphContainer<NL, EL, G> host) {
        if (host.getFp().contains(pattern.getFp())) {
            return new MatcherFast<NL, EL>(pattern, host).match() ? 0.0 : 1.0;
        } else {
            return 1.0;
        }
    }

    @Override
    public double maxDist() {
        return 1;
    }

    @Override
    public String getDescription() {
        return "SubIsoFpGGD";
    }

}
