package struclust.comparison;

import graph.Graph;
import match.MatcherFast;
import struclust.SharedMemorySCC;
import struclust.graph.GraphContainer;

/**
 * The value of this distance is <it>-1 * |pattern graph|^pow</it> if there
 * exists an embedding of the pattern graph in the target/host graph and 0 if
 * not. The distance is non-symmetric.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class WeightedSubIsoFpGGD<NL, EL, G extends Graph<NL, EL>> implements GraphGraphDistance<NL, EL, G> {

    private SharedMemorySCC<NL, EL, G> conf;
    private double pow;

    /**
     * Constructor
     * 
     * @param pow
     *            the distance is powered by this value
     * @param conf
     *            the configuration
     */
    public WeightedSubIsoFpGGD(double pow, SharedMemorySCC<NL, EL, G> conf) {
        this.pow = pow;
        this.conf = conf;
    }

    @Override
    public double calc(GraphContainer<NL, EL, G> pattern, GraphContainer<NL, EL, G> host) {
        assert pattern != null;
        assert host != null;
        
        if (host.getFp().contains(pattern.getFp())) {
            return new MatcherFast<NL, EL>(pattern, host).match() ? -1.0 * Math.pow(pattern.size(conf.gSize), pow) : 0.0;
        }else {
            return 0.0;
        }
    }

    @Override
    public double maxDist() {
        return 0.0;
    }

    @Override
    public String getDescription() {
        return "WeightedSubIsoFpGGD (pow: " + pow + ")";
    }
}
