package struclust.competingalgorithms.commonsio;

import graph.Graph;
import hsahn.comparison.kernel.graph.WeisfeilerLehmanSubtreeKernel;
import struclust.StructuralClusteringConf;
import struclust.comparison.GraphGraphDistance;
import struclust.comparison.NormKernelGGD;
import struclust.eval.EvalResult;

/**
 * Configuration for a {@link WrappedDbscan} clustering.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class DbscanConf<NL, EL, G extends Graph<NL, EL>> implements StructuralClusteringConf<NL, EL, G> {

    /**
     * The distance between the {@link Graph}s
     */
    public GraphGraphDistance<NL, EL, G> dist = new NormKernelGGD<>(new WeisfeilerLehmanSubtreeKernel<>(5));

    /**
     * The radius for core object density calculation
     */
    public double eps = 1;

    /**
     * The minimum number of objects in {@link #eps} radius for core objects
     */
    public int minPts = 10;

    @Override
    public void addAsMetadata(EvalResult result) {
        result.addMetadata("distance", dist);
        result.addMetadata("eps", eps);
        result.addMetadata("minPts", minPts);
    }
}
