package struclust.competingalgorithms.commonsio;

import graph.Graph;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.math3.ml.clustering.Clusterable;

import struclust.graph.GraphContainer;

/**
 * Wraps a graph into a {@link Clusterable}
 * 
 * @author Till Schäfer
 *
 */
public class WrappedGraphClusterable implements Clusterable {

    private double[] point;

    /**
     * Constructor
     * 
     * @param id
     *            the id of the wrapped {@link Graph}
     */
    public WrappedGraphClusterable(int id) {
        point = new double[] { id };
    }

    @Override
    public double[] getPoint() {
        return point;
    }

    /**
     * Get the id of the wrapped graph
     * 
     * @return the id of the wrapped graph
     */
    public int getId() {
        return (int) point[0];
    }

    /**
     * creates {@link WrappedGraphClusterable}s for each {@link GraphContainer}
     * in id2Graph. The ordering is preserved.
     * 
     * @param id2Graph
     * @return the wrapped graphs
     */
    public static List<WrappedGraphClusterable> wrap(List<? extends GraphContainer<?, ?, ?>> id2Graph) {
        ArrayList<WrappedGraphClusterable> retVal = new ArrayList<>(id2Graph.size());
        for (int id = 0; id < id2Graph.size(); id++) {
            retVal.add(new WrappedGraphClusterable(id));
        }
        return retVal;
    }

}
