package struclust.competingalgorithms.opensubspace;

import graph.Graph;
import hsahn.comparison.kernel.ExplicitMappingKernel;
import hsahn.datastructure.FeatureVector;
import hsahn.graph.GraphTools;
import hsahn.graph.LGraph;

import java.util.ArrayList;
import java.util.Collections;

/**
 * Uses an arbitrary {@link ExplicitMappingKernel} for {@link Graph}s to map a
 * {@link Graph} to a {@link FeatureVector}.<br>
 * 
 * <b>Important:</b> The {@link FeatureVector}s from multiple calls of
 * {@link ExplicitMappingKernel#computeExplicitMapping(java.util.Collection)}
 * must be comparable!
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class ExplicitKernelTFVF<NL, EL, G extends Graph<NL, EL>> implements ToFeatureVectorFunction<NL, EL, G> {

    private ExplicitMappingKernel<LGraph<NL, EL>, ? extends FeatureVector<? extends Object>> kernel;

    /**
     * Constructor
     * 
     * @param kernel
     *            the kernel to compute the explicit mapping with
     */
    public ExplicitKernelTFVF(ExplicitMappingKernel<LGraph<NL, EL>, ? extends FeatureVector<? extends Object>> kernel) {
        this.kernel = kernel;
    }

    @Override
    public FeatureVector<? extends Object> apply(G t) {

        LGraph<NL, EL> lineGraph = GraphTools.convert(t);
        ArrayList<? extends FeatureVector<? extends Object>> explicitMappingCollection = kernel
                .computeExplicitMapping(Collections.singleton(lineGraph));
        return explicitMappingCollection.get(0);
    }

    @Override
    public String getDescription() {
        return "ExplicitKernelTFVF (Kernel: " + kernel.toString() + ")";
    }

}
