package struclust.competingalgorithms.opensubspace;

import graph.Graph;
import i9.subspace.ken.ConfigReader;
import i9.subspace.ken.MineCLUS;
import struclust.StructuralClusteringConf;
import struclust.eval.EvalResult;

/**
 * Configuration for {@link WrappedMineClus} clustering.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class MineClusConf<NL, EL, G extends Graph<NL, EL>> implements StructuralClusteringConf<NL, EL, G> {

    /**
     * This transforms the graphs to a feature vector, that can be clustered
     * then.
     */
    public ToFeatureVectorFunction<NL, EL, G> toFeatureVectorFunction;

    /**
     * The number of clusters
     */
    public int numClusters = 10;

    /**
     * The minimum size of a cluster as fraction of the whole dataset.
     * 
     * The value must be in (0,1]
     */
    public double alpha = 0.001;
    /**
     * importance of the size of the subspace over the size of the cluster.
     * 
     * The value must be in (0,1]
     */
    public double beta = 0.1;
    /**
     * the distances between records in the same cluster in each relevant
     * dimension are bounded by w
     */
    public double w;
    /**
     * The number of optimization steps
     */
    public int iterations;

    @Override
    public void addAsMetadata(EvalResult result) {
        result.addMetadata("toFeatureVectorFunction", toFeatureVectorFunction);
        result.addMetadata("numClusters", numClusters);

    }

    /**
     * @return a configuration object for {@link MineCLUS}
     */
    public ConfigReader getConfigReader() {
        ConfigReader retVal = new ConfigReader();
        char configNameMarker = '#';
        String[] configArray = new String[14];

        configArray[0] = "#alpha";
        configArray[1] = Double.toString(alpha);
        configArray[2] = "#beta";
        configArray[3] = Double.toString(beta);
        configArray[4] = "#w";
        configArray[5] = Double.toString(w);
        // disable maximum outer iteration limitation
        configArray[6] = "#MAXOUT";
        configArray[7] = Integer.toString(0);
        // this is just for evaluation
        configArray[8] = "#nb";
        configArray[9] = Integer.toString(1);
        configArray[10] = "#k";
        configArray[11] = Integer.toString(numClusters);
        configArray[12] = "#tcr";
        configArray[13] = Integer.toString(iterations);

        retVal.addConfigFromArray(configArray, configNameMarker);

        return retVal;

    }

}
