package struclust.competingalgorithms.opensubspace;

import graph.Graph;
import struclust.StructuralClusteringConf;
import struclust.eval.EvalResult;

/**
 * Configuration for {@link WrappedProclus} clustering.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class ProclusConf<NL, EL, G extends Graph<NL, EL>> implements StructuralClusteringConf<NL, EL, G> {

    /**
     * This transforms the graphs to a feature vector, that can be clustered
     * then.
     */
    public ToFeatureVectorFunction<NL, EL, G> toFeatureVectorFunction;

    /**
     * The number of clusters
     */
    public int numClusters = 10;

    /**
     * The average number of features cosidered for a cluster
     */
    public int avgDimension = 10;

    @Override
    public void addAsMetadata(EvalResult result) {
        result.addMetadata("toFeatureVectorFunction", toFeatureVectorFunction);
        result.addMetadata("numClusters", numClusters);
        result.addMetadata("avgDimension", avgDimension);
    }

}
