package struclust.convergence;

import java.util.Collection;

import struclust.Cluster;

/**
 * Simple {@link ConvergenceCriteria} that assumes the algorithm to be
 * convergent, if the number of changes falls below a specified percentage of
 * elements
 * 
 * @author Till Schäfer
 *
 */
public class ChangesPercentageCC implements ConvergenceCriteria {

    private double percentage;

    /**
     * Constructor
     * 
     * @param percentage
     *            the percentage for convergence.
     */
    public ChangesPercentageCC(double percentage) {
        this.percentage = percentage;
    }

    @Override
    public boolean convergent(int changes, int totalElements, Collection<? extends Cluster<?, ?, ?>> clustering) {
        int criteria = (int) Math.floor(totalElements * percentage + 1);
        return changes < criteria;
    }

    @Override
    public void reset() {
        // nothing to do here
    }
    
    @Override
    public String getDescription() {
        return "ChangesPercentageCC " + percentage + "%";
    }

}
