package struclust.convergence;

import java.util.Collection;

import struclust.Cluster;

/**
 * Simple {@link ConvergenceCriteria} that uses a predefined number of
 * iterations. It mainly serves testing purposes.
 * 
 * @author Till Schäfer
 *
 */
public class FixedIterationNumberCC implements ConvergenceCriteria {

    private int numIterations;
    private int count = 0;

    /**
     * Constructor
     * 
     * @param numIterations
     *            the number of converget calls after which true is returned.
     */
    public FixedIterationNumberCC(int numIterations) {
        this.numIterations = numIterations;
    }

    @Override
    public String getDescription() {
        return "FixedIterationNumberCC(" + numIterations + ")";
    }

    @Override
    public boolean convergent(int changes, int totalElements, Collection<? extends Cluster<?, ?, ?>> clustering) {
        count++;
        return count > numIterations;
    }

    @Override
    public void reset() {
        count = 0;
    }

}
