package struclust.granularity;

import graph.Graph;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.tuple.Pair;

import struclust.Cluster;
import struclust.ClusterFactory;
import struclust.StructuralClustering;

/**
 * Merges the cluster pairs by putting their members together and re-mining the
 * representatives.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class ReMineMS<NL, EL, G extends Graph<NL, EL>> implements MergeStrategy<NL, EL, G> {

    @Override
    public String getDescription() {
        return "ReMineMS";
    }

    @Override
    public LinkedList<Cluster<NL, EL, G>> merge(List<Pair<Cluster<NL, EL, G>, Cluster<NL, EL, G>>> toMerge,
            List<Cluster<NL, EL, G>> allClusters, ClusterFactory<NL, EL, G> clusterFactory,
            StructuralClustering<NL, EL, G> parentClusterer) {

        HashSet<Cluster<NL, EL, G>> mergeCluster = new HashSet<>();
        mergeCluster.addAll(toMerge.stream().map(pair -> pair.getLeft()).collect(Collectors.toList()));
        mergeCluster.addAll(toMerge.stream().map(pair -> pair.getRight()).collect(Collectors.toList()));

        LinkedList<Cluster<NL, EL, G>> retVal = allClusters.stream().filter(c -> !mergeCluster.contains(c))
                .collect(Collectors.toCollection(LinkedList::new));
        LinkedList<Cluster<NL, EL, G>> mergedClusters = new LinkedList<>();
        for (Pair<Cluster<NL, EL, G>, Cluster<NL, EL, G>> pair : toMerge) {
            Cluster<NL, EL, G> mc = clusterFactory.emptyCluster();
            mc.addAll(pair.getLeft());
            mc.addAll(pair.getRight());

            mergedClusters.add(mc);
        }

        parentClusterer.updateReps(mergedClusters);
        retVal.addAll(mergedClusters);

        return retVal;
    }

}
