package struclust.graph.generators;

import graph.Graph;

import java.util.List;

/**
 * Interface for {@link Graph} generators which exhibit a clustering structure
 * from the graph generation process.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public interface ClusteredGraphGenerator<NL, EL, G extends Graph<NL, EL>> extends RandomGraphGenerator<NL, EL, G> {
    /**
     * Generate some {@link Graph}s together with their clustering structure
     * 
     * @param num
     *            the number of graphs to generate
     * @return the generated clusters of graphs
     */
    List<List<G>> createGraphsClustered(int num);
}
