package struclust.mcs;

import graph.Graph;
import hsahn.graph.GraphTools;
import hsahn.graph.LGraph;
import hsahn.mcs.MCSAlgorithm;

/**
 * Computes a maximum common (not-connected) induced subgraph.
 * 
 * The implementation is a wrapper around the HSAHN implementation of Nils
 * Kriege.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class McisHsahnMSA<NL, EL, G extends Graph<NL, EL>> implements McsSizeAlgorithm<NL, EL, G> {
    private static final MCSAlgorithm algo = new MCSAlgorithm.MaximumClique();

    @Override
    public boolean isConnected() {
        return false;
    }

    @Override
    public boolean isInduced() {
        return true;
    }

    @Override
    public String getDescription() {
        return "McisHsahnMSA";
    }

    @Override
    public int calc(G graph1, G graph2) {
        LGraph<NL, EL> lg1 = GraphTools.convert(graph1);
        LGraph<NL, EL> lg2 = GraphTools.convert(graph2);
        
        int mcs = algo.computeMCS(lg1, lg2);

        return mcs;
    }

}
