package struclust.mcs;

import graph.Graph;

/**
 * Algorithm, that computes the size of a maximum common subgraph (MCS) of two
 * {@link Graph}s.
 * 
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public interface McsSizeAlgorithm<NL, EL, G extends Graph<NL, EL>> extends McsAlgorithm {
    /**
     * Calculate the size of the maximum common subgraph (MCS).
     * 
     * @param graph1
     *            the first {@link Graph}
     * @param graph2
     *            the second {@link Graph}
     * @return the size of the MCS
     */
    public int calc(G graph1, G graph2);
}
