package struclust.mcs;

import graph.Graph;

/**
 * Algorithm, that calculates, if there exists a maximum common subgraph (MCS)
 * which is larger than a specified minimum size.
 * 
 * @author Till Schäfer
 * 
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public interface McsSizeDecisionAlgorithm<NL, EL, G extends Graph<NL, EL>> extends McsAlgorithm {
    /**
     * Calculates, whether there exist a maximum common subgraph (MCS) with at
     * least minSize.
     * 
     * @param graph1
     *            the first {@link Graph}
     * @param graph2
     *            the second {@link Graph}
     * @param minSize
     *            the minimum size
     * @return whether there exist a MCS with at least minSize
     */
    public boolean calc(G graph1, G graph2, int minSize);
}
