package struclust.mining;

import graph.Graph;

import java.util.List;

import struclust.SharedMemorySCC;
import struclust.datastructures.MutableInt;
import struclust.graph.GraphContainer;

/**
 * Very simple support counting without any optimizations.
 * 
 * @author Till Schäfer
 *
 * @param <NL>
 *            the node label type
 * @param <EL>
 *            the edge label type
 * @param <G>
 *            the graph type
 */
public class SimpleSC<NL, EL, G extends Graph<NL, EL>> extends AbstractSC<NL, EL, G> {

    /**
     * Constructor
     * 
     * @param parallelism
     *            the parallelism (see {@link SharedMemorySCC#parallelism}
     */
    public SimpleSC(MutableInt parallelism) {
        super(parallelism);
    }

    @Override
    public boolean supported(List<GraphContainer<NL, EL, G>> graphs, G pattern, double minSup, int numHypothesis,
            boolean parallel) {
        double sup = (double) numSuportedGraphs(graphs, pattern, parallel) / graphs.size();

        return sup >= minSup;
    }

    @Override
    public double support(List<GraphContainer<NL, EL, G>> graphs, G pattern, boolean parallel) {
        return (double) numSuportedGraphs(graphs, pattern, parallel) / graphs.size();
    }

    @Override
    public String getDescription() {
        return "SimpleSC";
    }

}
