package struclust.tools.samplemembers;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.JCommander;

/**
 * Samples the cluster members from a GML-Clustering-File with a given
 * probability.
 * 
 * @author Till Schäfer
 */
public class SampleMembers {
    private final static Random rand = new Random();
    /**
     * Main-Method
     * 
     * @param args
     *            see {@link Parameters} for further information
     */
    public static void main(String[] args) {
        Parameters param = new Parameters();
        new JCommander(param, args);

        try (FileReader input = new FileReader(param.inFile);
                BufferedReader reader = new BufferedReader(input);
                FileWriter output = new FileWriter(param.outFile);
                BufferedWriter writer = new BufferedWriter(output);) {
            boolean memberLine = false;
            boolean graphLine = false;
            int openBlocksInsideMember = 0;
            int openBlocksInsideGraph = 0;
            for (String line = reader.readLine(); line != null; line = reader.readLine()) {
                if (!memberLine) {
                    if (line.contains("members")) {
                        memberLine = true;
                        openBlocksInsideMember = 0;
                    }
                    writer.write(line + System.getProperty("line.separator"));
                } else {
                    openBlocksInsideMember += StringUtils.countMatches(line, "[");
                    openBlocksInsideMember -= StringUtils.countMatches(line, "]");
                    if (openBlocksInsideMember < 0) {
                        memberLine = false;
                        writer.write(line + System.getProperty("line.separator"));
                        continue;
                    }
                    
                    if (!graphLine) {
                        boolean containedInSample = rand.nextDouble() < param.prob;
                        if (line.contains("graph") && containedInSample) {
                            graphLine = true;
                            openBlocksInsideGraph = 0;
                            writer.write(line + System.getProperty("line.separator"));
                        }
                    } else {
                        openBlocksInsideGraph += StringUtils.countMatches(line, "[");
                        openBlocksInsideGraph -= StringUtils.countMatches(line, "]");
                        
                        writer.write(line + System.getProperty("line.separator"));
                        if (openBlocksInsideGraph < 0) {
                            graphLine = false;
                        }
                    }
                }
            }
        } catch (IOException e) {
            System.out.println("Could not read file");
            e.printStackTrace();
        }
    }

}
