package struclust.tools.splitclusters;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import com.beust.jcommander.JCommander;

/**
 * Splits an GML-Clustering-File into several small files, each containing a
 * single cluster.
 * 
 * @author Till Schäfer
 *
 */
public class SplitClusters {

    /**
     * Main-Method
     * 
     * @param args
     *            see {@link Parameters} for further information
     */
    public static void main(String[] args) {
        Parameters param = new Parameters();
        new JCommander(param, args);

        try (FileReader input = new FileReader(param.inFile); BufferedReader reader = new BufferedReader(input)) {
            FileWriter output = null;
            BufferedWriter writer = null;
            for (String line = reader.readLine(); line != null; line = reader.readLine()) {
                if (line.contains("cluster")) {
                    if (writer != null) {
                        writer.close();
                        output.close();
                    }
                    try {
                        // read a line ahead to determine the cluster ID
                        String nextLine = reader.readLine();
                        if (nextLine == null || !nextLine.contains("id")) {
                            throw new InvalidFormatException("Cluster may not be empty");
                        }
                        int clusterId;
                        try {
                            clusterId = Integer.parseInt(nextLine.replaceFirst("id ", ""));
                        } catch (NumberFormatException e) {
                            throw new InvalidFormatException("Cluster ID cannot be parsed");
                        }

                        output = new FileWriter(param.outFilePrefix + clusterId + ".gml");
                        writer = new BufferedWriter(output);

                        writer.write(line + System.getProperty("line.separator"));
                        line = nextLine;
                    } catch (IOException e) {
                        System.out.println("Could not write file");
                        e.printStackTrace();
                        writer.close();
                        output.close();
                        System.exit(1);
                    }
                }

                writer.write(line + System.getProperty("line.separator"));
            }
            writer.close();
            output.close();
        } catch (IOException e) {
            System.out.println("Could not read file");
            e.printStackTrace();
        }
    }
}
