/*
 * Scaffold Hunter
 * Copyright (C) 2006-2008 PG504
 * Copyright (C) 2010-2011 PG552
 * See README.txt in the root directory of the Scaffold Hunter source tree
 * for details.
 *
 * Scaffold Hunter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Scaffold Hunter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package struclust.util;

import org.openscience.cdk.aromaticity.Aromaticity;
import org.openscience.cdk.aromaticity.ElectronDonation;
import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.graph.Cycles;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.smiles.SmilesGenerator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Wrapper class for generating canonical SMILES strings. Should be used instead
 * of the inner CDK smiles generator, to ensure the same configuration is used
 * to generate the smiles strings.
 * 
 * @author Philipp Lewe, Till Schäfer
 * 
 */
public class CanonicalSmilesGenerator {
    private static final Logger logger = LoggerFactory.getLogger(CanonicalSmilesGenerator.class);
    private static SmilesGenerator smigen = SmilesGenerator.unique();

    /**
     * @param mol
     *            the molecule of type <code>IAtomContainer</code>
     * @return the canonical smiles string
     */
    public static String createSMILES(IAtomContainer mol) {
        Aromaticity aromaticity = new Aromaticity(ElectronDonation.daylight(), Cycles.all());
        try {
            aromaticity.apply(mol);
        } catch (CDKException e1) {
            e1.printStackTrace();
            return "";
        }
        smigen.aromatic();
        try {
            return smigen.create(mol);
        } catch (Exception e) {
            logger.error("Smiles generation failed", e);
            // We catch all Exceptions here to be on the save side, since to
            // many things
            // have already gone wrong with SMILES generation
            return "";
        }
    }
}
