package struclust.util;

import java.util.concurrent.ForkJoinPool;

/**
 * Class for convenient methods regarding concurrency
 * 
 * @author Till Schäfer
 */
public class Concurrent {
    /**
     * Returns the rawValue for each value above zero and the number of
     * available processors otherwise.
     * 
     * @param rawValue
     *            the raw value
     * @return the effective parallelism.
     */
    public static int getEffectiveParallelism(int rawValue) {
        if (rawValue < 1) {
            return Runtime.getRuntime().availableProcessors();
        } else {
            return rawValue;
        }
    }

    /**
     * Sets the common ForkJoinPool parallelism (used for parallel streams) to
     * the desired level.
     * 
     * <br>
     * <strong>Attention</strong>: You must call this method before any parallel
     * stream operation is executed. Calling this method afterwards will have no
     * effect.
     * 
     * @param maxThreads
     *            the maximum number of threads running in parallel. Note: this
     *            is different from the {@link ForkJoinPool} parallelism value,
     *            because the {@link ForkJoinPool} only considers additional
     *            threads additionally to the main thread. This value is an
     *            absolute value including the main thread.
     */
    public static void setCommonFJPParallelism(int maxThreads) {
        int additionalThreads = maxThreads - 1;
        System.setProperty("java.util.concurrent.ForkJoinPool.common.parallelism", String.valueOf(additionalThreads));
        assert ForkJoinPool.getCommonPoolParallelism() == additionalThreads;
    }
}
