package struclust.graph;

import java.util.LinkedList;

import org.junit.Assert;

import graph.DefaultGraph;
import graph.Graph;
import graph.Node;

/**
 * JUnit Tests for {@link Graphs}
 * 
 * @author Till Schäfer
 *
 */
public class GraphsTest {
    /**
     * Tests the Method {@link Graphs#distinctGraphs(java.util.List)} by
     * simple example
     */
    @org.junit.Test
    public void distinctGraphsTest() {
        Graph<String, String> graph = new DefaultGraph<String, String>();
        Node<String, String> newNode1 = graph.addNode("A");
        Node<String, String> newNode2 = graph.addNode("B");
        Node<String, String> newNode3 = graph.addNode("C");
        Node<String, String> newNode4 = graph.addNode("D");
        Node<String, String> newNode5 = graph.addNode("D");
        Node<String, String> newNode6 = graph.addNode("E");

        graph.addEdge(newNode1, newNode2, "-");
        graph.addEdge(newNode1, newNode4, "-");
        graph.addEdge(newNode2, newNode3, "-");
        graph.addEdge(newNode2, newNode4, "-");
        graph.addEdge(newNode2, newNode5, "-");
        graph.addEdge(newNode3, newNode5, "-");
        graph.addEdge(newNode4, newNode5, "-");
        graph.addEdge(newNode3, newNode6, "-");

        Graph<String, String> graph2 = new DefaultGraph<String, String>(graph);
        graph2.removeNode(graph2.getNode(0));

        Graph<String, String> graph3 = new DefaultGraph<String, String>(graph);
        graph3.removeNode(graph3.getNode(1));

        Graph<String, String> graph4 = new DefaultGraph<String, String>(graph);
        graph4.removeNode(graph4.getNode(2));

        LinkedList<GraphContainer<String, String, Graph<String, String>>> graphs = new LinkedList<>();
        graphs.add(new GraphContainer<>(graph2));
        graphs.add(new GraphContainer<>(graph));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graph)));
        graphs.add(new GraphContainer<>(graph3));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graph3)));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graph)));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graph)));
        graphs.add(new GraphContainer<>(graph4));

        Graphs.distinctGraphs(graphs);

        Assert.assertEquals(4, graphs.size());
    }

    /**
     * Tests the Method {@link Graphs#filterSubgraphs(java.util.List)} by simple
     * example
     */
    @org.junit.Test
    public void filterSubgraphsTest() {
        Graph<String, String> graphA = new DefaultGraph<String, String>();
        Node<String, String> newNode1 = graphA.addNode("A");
        Node<String, String> newNode2 = graphA.addNode("B");
        Node<String, String> newNode3 = graphA.addNode("C");
        Node<String, String> newNode4 = graphA.addNode("D");
        Node<String, String> newNode5 = graphA.addNode("D");
        Node<String, String> newNode6 = graphA.addNode("E");

        graphA.addEdge(newNode1, newNode2, "-");
        graphA.addEdge(newNode1, newNode4, "-");
        graphA.addEdge(newNode2, newNode3, "-");
        graphA.addEdge(newNode2, newNode4, "-");
        graphA.addEdge(newNode2, newNode5, "-");
        graphA.addEdge(newNode3, newNode5, "-");
        graphA.addEdge(newNode4, newNode5, "-");
        graphA.addEdge(newNode3, newNode6, "-");

        // subgraphs of graphA
        Graph<String, String> graph2 = new DefaultGraph<String, String>(graphA);
        graph2.removeNode(graph2.getNode(0));
        Graph<String, String> graph3 = new DefaultGraph<String, String>(graphA);
        graph3.removeNode(graph3.getNode(1));
        Graph<String, String> graph4 = new DefaultGraph<String, String>(graphA);
        graph4.removeNode(graph4.getNode(2));

        Graph<String, String> graphB = new DefaultGraph<String, String>();
        newNode1 = graphB.addNode("A");
        newNode2 = graphB.addNode("B");
        newNode3 = graphB.addNode("C");
        newNode4 = graphB.addNode("X");
        newNode5 = graphB.addNode("D");
        newNode6 = graphB.addNode("E");
        
        graphB.addEdge(newNode1, newNode2, "-");
        graphB.addEdge(newNode1, newNode4, "-");
        graphB.addEdge(newNode2, newNode3, "-");
        graphB.addEdge(newNode2, newNode4, "-");
        graphB.addEdge(newNode2, newNode5, "-");
        graphB.addEdge(newNode3, newNode5, "-");
        graphB.addEdge(newNode4, newNode5, "-");
        graphB.addEdge(newNode3, newNode6, "-");
        
        
        LinkedList<GraphContainer<String, String, Graph<String, String>>> graphs = new LinkedList<>();
        graphs.add(new GraphContainer<>(graph2));
        graphs.add(new GraphContainer<>(graphA));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graphA)));
        graphs.add(new GraphContainer<>(graph3));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graph3)));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graphA)));
        graphs.add(new GraphContainer<>(graphB));
        graphs.add(new GraphContainer<>(new DefaultGraph<>(graphA)));
        graphs.add(new GraphContainer<>(graph4));

        Graphs.filterSubgraphs(graphs);

        Assert.assertEquals(2, graphs.size());
    }
}
