package struclust.mcs;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import graph.DefaultGraph;
import graph.DefaultGraphFactory;
import graph.Node;

/**
 * JUnit test class for {@link MccisKriegeMEA}
 * 
 * @author Till Schäfer
 */
public class MccisExampleTests {
    /**
     * Test by example with mcs size 0
     */
    @org.junit.Test
    public void mcsSize0Test() {
        DefaultGraph<String, String> g1 = new DefaultGraph<>();
        DefaultGraph<String, String> g2 = new DefaultGraph<>();

        Node<String, String> a11 = g1.addNode("A");
        Node<String, String> a12 = g1.addNode("A");
        g1.addNode("A");
        g1.addNode("A");
        g1.addNode("B");
        g1.addEdge(a11, a12, "X");

        Node<String, String> a21 = g2.addNode("C");
        Node<String, String> a22 = g2.addNode("C");
        g2.addNode("C");
        g2.addEdge(a21, a22, "X");

        McsEnumAlgorithm<String, String, DefaultGraph<String, String>> algoE = new MccisKriegeMEA<>(
                new DefaultGraphFactory<>());

        List<DefaultGraph<String, String>> mcciss = algoE.calc(g1, g2);

        assertEquals(0, mcciss.size());

        for (McsSizeAlgorithm<String, String, DefaultGraph<String, String>> algoS : getMccisMsaAlgos()) {
            assertEquals(algoS.getDescription() + " wrong mcs size", 0, algoS.calc(g1, g2));
        }

        McsSizeDecisionAlgorithm<String, String, DefaultGraph<String, String>> mccisKriegeColor2MSDA = new MccisKriegeColor2MSDA<>();
        assertTrue(mccisKriegeColor2MSDA.calc(g1, g2, 0));
        assertFalse(mccisKriegeColor2MSDA.calc(g1, g2, 1));

    }

    /**
     * Test by example with mcs size 1
     */
    @org.junit.Test
    public void mcsSize2Test() {
        DefaultGraph<String, String> g1 = new DefaultGraph<>();
        DefaultGraph<String, String> g2 = new DefaultGraph<>();

        Node<String, String> a11 = g1.addNode("A");
        Node<String, String> a12 = g1.addNode("A");
        g1.addNode("A");
        g1.addNode("A");
        g1.addNode("B");
        g1.addEdge(a11, a12, "X");

        Node<String, String> a21 = g2.addNode("A");
        Node<String, String> a22 = g2.addNode("A");
        g2.addNode("B");
        g2.addEdge(a21, a22, "X");

        McsEnumAlgorithm<String, String, DefaultGraph<String, String>> algoE = new MccisKriegeMEA<>(
                new DefaultGraphFactory<>());

        List<DefaultGraph<String, String>> mcciss = algoE.calc(g1, g2);

        assertEquals(1, mcciss.size());

        assertEquals(2, mcciss.get(0).getNodeCount());
        assertEquals(1, mcciss.get(0).getEdgeCount());

        for (McsSizeAlgorithm<String, String, DefaultGraph<String, String>> algoS : getMccisMsaAlgos()) {
            assertEquals(algoS.getDescription() + " wrong mcs size", 2, algoS.calc(g1, g2));
        }

        McsSizeDecisionAlgorithm<String, String, DefaultGraph<String, String>> mccisKriegeColor2MSDA = new MccisKriegeColor2MSDA<>();
        assertTrue(mccisKriegeColor2MSDA.calc(g1, g2, 2));
        assertFalse(mccisKriegeColor2MSDA.calc(g1, g2, 3));

    }

    /**
     * Test by example with mcs size 4
     */
    @org.junit.Test
    public void mcsSize4Test() {
        DefaultGraph<String, String> ga = new DefaultGraph<>();
        DefaultGraph<String, String> gb = new DefaultGraph<>();
    
        Node<String, String> a3 = ga.addNode("a");
        Node<String, String> a4 = ga.addNode("a");
        Node<String, String> a6 = ga.addNode("a");
        Node<String, String> a7 = ga.addNode("a");
    
        ga.addEdge(a3, a4, "");
        ga.addEdge(a3, a6, "");
        ga.addEdge(a3, a7, "");
        ga.addEdge(a4, a6, "");
        ga.addEdge(a4, a7, "");
        ga.addEdge(a6, a7, "");
    
        Node<String, String> b1 = gb.addNode("b");
        Node<String, String> b2 = gb.addNode("a");
        Node<String, String> b3 = gb.addNode("a");
        Node<String, String> b4 = gb.addNode("a");
        Node<String, String> b5 = gb.addNode("a");
        Node<String, String> b6 = gb.addNode("a");
        Node<String, String> b7 = gb.addNode("a");
        Node<String, String> b8 = gb.addNode("a");
        Node<String, String> b9 = gb.addNode("b");
        Node<String, String> b10 = gb.addNode("x");
        Node<String, String> b11 = gb.addNode("x");
        Node<String, String> b12 = gb.addNode("b");
    
        gb.addEdge(b1, b2, "");
        gb.addEdge(b1, b3, "");
        gb.addEdge(b2, b3, "");
        gb.addEdge(b2, b4, "");
        gb.addEdge(b2, b5, "");
        gb.addEdge(b3, b4, "");
        gb.addEdge(b3, b5, "");
        gb.addEdge(b3, b8, "");
        gb.addEdge(b4, b5, "");
        gb.addEdge(b4, b12, "");
        gb.addEdge(b5, b8, "");
        gb.addEdge(b5, b12, "");
        gb.addEdge(b6, b7, "");
        gb.addEdge(b6, b8, "");
        gb.addEdge(b7, b8, "");
        gb.addEdge(b7, b9, "");
        gb.addEdge(b8, b9, "");
        gb.addEdge(b8, b10, "");
        gb.addEdge(b9, b10, "");
        gb.addEdge(b10, b11, "");
    
        McsEnumAlgorithm<String, String, DefaultGraph<String, String>> algo = new MccisKriegeMEA<>(
                new DefaultGraphFactory<>());
    
        List<DefaultGraph<String, String>> mcciss = algo.calc(ga, gb);
    
        assertEquals(1, mcciss.size());
    
        assertEquals(4, mcciss.get(0).getNodeCount());
        assertEquals(6, mcciss.get(0).getEdgeCount());
    
        for (McsSizeAlgorithm<String, String, DefaultGraph<String, String>> algoS : getMccisMsaAlgos()) {
            assertEquals(algoS.getDescription() + " wrong mcs size", 4, algoS.calc(ga, gb));
        }
    
        McsSizeDecisionAlgorithm<String, String, DefaultGraph<String, String>> mccisKriegeColor2MSDA = new MccisKriegeColor2MSDA<>();
        assertTrue(mccisKriegeColor2MSDA.calc(ga, gb, 4));
        assertFalse(mccisKriegeColor2MSDA.calc(ga, gb, 6));
    }

    /**
     * Test by example with mcs size 6
     */
    @org.junit.Test
    public void mcsSize6Test() {
        DefaultGraph<String, String> ga = new DefaultGraph<>();
        DefaultGraph<String, String> gb = new DefaultGraph<>();

        Node<String, String> a1 = ga.addNode("b");
        Node<String, String> a2 = ga.addNode("b");
        Node<String, String> a3 = ga.addNode("a");
        Node<String, String> a4 = ga.addNode("a");
        Node<String, String> a5 = ga.addNode("b");
        Node<String, String> a6 = ga.addNode("a");
        Node<String, String> a7 = ga.addNode("a");
        Node<String, String> a8 = ga.addNode("b");
        Node<String, String> a9 = ga.addNode("x");

        ga.addEdge(a1, a3, "");
        ga.addEdge(a2, a3, "");
        ga.addEdge(a2, a4, "");
        ga.addEdge(a3, a4, "");
        ga.addEdge(a3, a6, "");
        ga.addEdge(a3, a7, "");
        ga.addEdge(a4, a5, "");
        ga.addEdge(a4, a6, "");
        ga.addEdge(a4, a7, "");
        ga.addEdge(a6, a7, "");
        ga.addEdge(a6, a8, "");
        ga.addEdge(a7, a8, "");
        ga.addEdge(a8, a9, "");

        Node<String, String> b1 = gb.addNode("b");
        Node<String, String> b2 = gb.addNode("a");
        Node<String, String> b3 = gb.addNode("a");
        Node<String, String> b4 = gb.addNode("a");
        Node<String, String> b5 = gb.addNode("a");
        Node<String, String> b6 = gb.addNode("a");
        Node<String, String> b7 = gb.addNode("a");
        Node<String, String> b8 = gb.addNode("a");
        Node<String, String> b9 = gb.addNode("b");
        Node<String, String> b10 = gb.addNode("x");
        Node<String, String> b11 = gb.addNode("x");
        Node<String, String> b12 = gb.addNode("b");

        gb.addEdge(b1, b2, "");
        gb.addEdge(b1, b3, "");
        gb.addEdge(b2, b3, "");
        gb.addEdge(b2, b4, "");
        gb.addEdge(b2, b5, "");
        gb.addEdge(b3, b4, "");
        gb.addEdge(b3, b5, "");
        gb.addEdge(b3, b8, "");
        gb.addEdge(b4, b5, "");
        gb.addEdge(b4, b12, "");
        gb.addEdge(b5, b8, "");
        gb.addEdge(b5, b12, "");
        gb.addEdge(b6, b7, "");
        gb.addEdge(b6, b8, "");
        gb.addEdge(b7, b8, "");
        gb.addEdge(b7, b9, "");
        gb.addEdge(b8, b9, "");
        gb.addEdge(b8, b10, "");
        gb.addEdge(b9, b10, "");
        gb.addEdge(b10, b11, "");

        McsEnumAlgorithm<String, String, DefaultGraph<String, String>> algo = new MccisKriegeMEA<>(
                new DefaultGraphFactory<>());

        List<DefaultGraph<String, String>> mcciss = algo.calc(ga, gb);

        assertEquals(2, mcciss.size());

        Set<Integer> edgeCounts = new TreeSet<>();
        for (DefaultGraph<String, String> g : mcciss) {
            assertEquals(6, g.getNodeCount());
            edgeCounts.add(g.getEdgeCount());
        }

        Set<Integer> expEdgeCounts = new TreeSet<>();
        expEdgeCounts.add(6);
        expEdgeCounts.add(10);
        assertEquals(expEdgeCounts, edgeCounts);

        for (McsSizeAlgorithm<String, String, DefaultGraph<String, String>> algoS : getMccisMsaAlgos()) {
            assertEquals(algoS.getDescription() + " wrong mcs size", 6, algoS.calc(ga, gb));
        }

        McsSizeDecisionAlgorithm<String, String, DefaultGraph<String, String>> mccisKriegeColor2MSDA = new MccisKriegeColor2MSDA<>();
        assertTrue(mccisKriegeColor2MSDA.calc(ga, gb, 6));
        assertFalse(mccisKriegeColor2MSDA.calc(ga, gb, 7));
    }

    /**
     * @return
     */
    private ArrayList<McsSizeAlgorithm<String, String, DefaultGraph<String, String>>> getMccisMsaAlgos() {
        ArrayList<McsSizeAlgorithm<String, String, DefaultGraph<String, String>>> algoSs = new ArrayList<>();
        algoSs.add(new MccisKriegeMSA<>());
        algoSs.add(new MccisKriegeColorMSA<>());
        algoSs.add(new MccisKriegeColor2MSA<>());
        return algoSs;
    }
}
