package struclust.mining;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import graph.DefaultEdge;
import graph.DefaultGraph;
import graph.DefaultNode;
import graph.LabelEdge;

/**
 * JUnit Test Class for {@link LabelEdge}
 * 
 * @author Till Schäfer
 */
public class LabelEdgeTest {
    /**
     * Tests the equals method based on string labels. (only positive)
     */
    @org.junit.Test
    public void EqualsTest() {
        DefaultGraph<String,String> g = new DefaultGraph<String,String>();
        DefaultNode<String,String> a1 = g.addNode("a");
        DefaultNode<String,String> a2 = g.addNode("a");
        DefaultNode<String,String> b1 = g.addNode("b");
        DefaultNode<String,String> b2 = g.addNode("b");
        DefaultEdge<String,String> e1 = g.addEdge(a1, b1, "c");
        DefaultEdge<String,String> e2 = g.addEdge(a2, b2, "c");
        
        
        LabelEdge<String,String> le1 = new LabelEdge<String,String>(e1);
        LabelEdge<String,String> le2 = new LabelEdge<String,String>(e2);
        assertEquals(le1, le1);
        assertEquals(le1, le2);
    }
    
    /**
     * Tests the equals method for reversed labeled edges based on string labels. (only positive)
     */
    @org.junit.Test
    public void ReverseEqualsTest() {
        DefaultGraph<String,String> g = new DefaultGraph<String,String>();
        DefaultNode<String,String> a1 = g.addNode("a");
        DefaultNode<String,String> a2 = g.addNode("a");
        DefaultNode<String,String> b1 = g.addNode("b");
        DefaultNode<String,String> b2 = g.addNode("b");
        DefaultEdge<String,String> e1 = g.addEdge(a1, b1, "c");
        DefaultEdge<String,String> e2 = g.addEdge(b2, a2, "c");
        
        
        LabelEdge<String,String> le1 = new LabelEdge<String,String>(e1);
        LabelEdge<String,String> le2 = new LabelEdge<String,String>(e2);
        assertEquals(le1, le2);
    }
    
    /**
     * Tests the equals method based on string labels. (only negative)
     */
    @org.junit.Test
    public void NotEqualsTest() {
        DefaultGraph<String,String> g = new DefaultGraph<String,String>();
        DefaultNode<String,String> a1 = g.addNode("a");
        DefaultNode<String,String> a2 = g.addNode("a");
        DefaultNode<String,String> b1 = g.addNode("b");
        DefaultEdge<String,String> e1 = g.addEdge(a1, b1, "c");
        DefaultEdge<String,String> e2 = g.addEdge(a1, b1, "x");
        DefaultEdge<String,String> e3 = g.addEdge(a1, a2, "c");
        
        
        LabelEdge<String,String> le1 = new LabelEdge<String,String>(e1);
        LabelEdge<String,String> le2 = new LabelEdge<String,String>(e2);
        LabelEdge<String,String> le3 = new LabelEdge<String,String>(e3);
        assertNotEquals(le1, le2);
        assertNotEquals(le1, le3);
    }
    
    /**
     * Tests if two different LabelEdge<String,String>s have the same hashCode if they are equal. 
     */
    @org.junit.Test
    public void hashCodeTest() {
        DefaultGraph<String,String> g = new DefaultGraph<String,String>();
        DefaultNode<String,String> a1 = g.addNode("a");
        DefaultNode<String,String> a2 = g.addNode("a");
        DefaultNode<String,String> b1 = g.addNode("b");
        DefaultNode<String,String> b2 = g.addNode("b");
        DefaultEdge<String,String> e1 = g.addEdge(a1, b1, "c");
        DefaultEdge<String,String> e2 = g.addEdge(a2, b2, "c");
        
        
        LabelEdge<String,String> le1 = new LabelEdge<String,String>(e1);
        LabelEdge<String,String> le2 = new LabelEdge<String,String>(e2);
        assertEquals(le1.hashCode(), le1.hashCode());
        assertEquals(le1.hashCode(), le2.hashCode());
    }
    
    /**
     * Tests the equals method for reversed labeled edges based on string labels. (only positive)
     */
    @org.junit.Test
    public void ReverseHashCodeTest() {
        DefaultGraph<String,String> g = new DefaultGraph<String,String>();
        DefaultNode<String,String> a1 = g.addNode("a");
        DefaultNode<String,String> a2 = g.addNode("a");
        DefaultNode<String,String> b1 = g.addNode("b");
        DefaultNode<String,String> b2 = g.addNode("b");
        DefaultEdge<String,String> e1 = g.addEdge(a1, b1, "c");
        DefaultEdge<String,String> e2 = g.addEdge(b2, a2, "c");
        
        
        LabelEdge<String,String> le1 = new LabelEdge<String,String>(e1);
        LabelEdge<String,String> le2 = new LabelEdge<String,String>(e2);
        assertEquals(le1.hashCode(), le2.hashCode());
    }
}
