package struclust.mining;

import graph.DefaultGraph;
import graph.Node;
import index.features.FingerprintBuilder;

import java.util.LinkedList;
import java.util.List;

import org.junit.Assert;

import struclust.datastructures.MutableInt;
import struclust.graph.GraphContainer;

/**
 * Tests for the class {@link PrefilteredSC}
 * 
 * @author Till Schäfer
 */
public class PrefilteredSCTest {
    /**
     * Simple test for two supported structures
     */
    @org.junit.Test
    public void twoSupportedStructuresTest() {
        DefaultGraph<String, String> graph1 = new DefaultGraph<String, String>();
        Node<String, String> node1 = graph1.addNode("A");
        Node<String, String> node2 = graph1.addNode("B");
        Node<String, String> node3 = graph1.addNode("C");
        Node<String, String> node4 = graph1.addNode("D");
        Node<String, String> node5 = graph1.addNode("D");

        graph1.addEdge(node1, node2, "-");
        graph1.addEdge(node1, node4, "-");
        graph1.addEdge(node2, node3, "-");
        graph1.addEdge(node2, node4, "-");
        graph1.addEdge(node2, node5, "-");
        graph1.addEdge(node3, node5, "-");
        graph1.addEdge(node4, node5, "-");

        DefaultGraph<String, String> graph2 = new DefaultGraph<String, String>();
        Node<String, String> node6 = graph2.addNode("A");
        Node<String, String> node7 = graph2.addNode("B");
        Node<String, String> node8 = graph2.addNode("C");
        Node<String, String> node9 = graph2.addNode("D");

        graph2.addEdge(node7, node8, "-");
        graph2.addEdge(node8, node9, "-");
        graph2.addEdge(node9, node7, "-");
        graph2.addEdge(node6, node7, "-");
        graph2.addEdge(node6, node8, "-");

        DefaultGraph<String, String> pattern = new DefaultGraph<String, String>();
        Node<String, String> node10 = pattern.addNode("B");
        Node<String, String> node11 = pattern.addNode("C");
        Node<String, String> node12 = pattern.addNode("D");

        pattern.addEdge(node10, node11, "-");
        pattern.addEdge(node11, node12, "-");
        pattern.addEdge(node12, node10, "-");

        FingerprintBuilder fpBuilder = new FingerprintBuilder(2048);
        List<GraphContainer<String, String, DefaultGraph<String, String>>> graphs = new LinkedList<>();
        GraphContainer<String, String, DefaultGraph<String, String>> gc1 = new GraphContainer<>(graph1);
        gc1.setFp(fpBuilder.getFingerprint(graph1));
        graphs.add(gc1);
        GraphContainer<String, String, DefaultGraph<String, String>> gc2 = new GraphContainer<>(graph2);
        gc2.setFp(fpBuilder.getFingerprint(graph2));
        graphs.add(gc2);

        PrefilteredSC<String, String, DefaultGraph<String, String>> sc = new PrefilteredSC<>(fpBuilder,
                new MutableInt());

        Assert.assertEquals("Suport is not equal to 1.0", 1.0, sc.support(graphs, pattern), 0.001);
        Assert.assertTrue("pattern is not supported, while it should be", sc.supported(graphs, pattern, 0.6));
    }

    /**
     * Simple test for two structures, one supported, and one not.
     */
    @org.junit.Test
    public void oneSupportedoneUnsupportedStructureTest() {
        DefaultGraph<String, String> graph1 = new DefaultGraph<String, String>();
        Node<String, String> node1 = graph1.addNode("A");
        Node<String, String> node2 = graph1.addNode("B");
        Node<String, String> node3 = graph1.addNode("C");
        Node<String, String> node4 = graph1.addNode("D");
        Node<String, String> node5 = graph1.addNode("D");

        graph1.addEdge(node1, node2, "-");
        graph1.addEdge(node1, node4, "-");
        graph1.addEdge(node2, node3, "=");
        graph1.addEdge(node2, node4, "-");
        graph1.addEdge(node2, node5, "-");
        graph1.addEdge(node3, node5, "-");
        graph1.addEdge(node4, node5, "-");

        DefaultGraph<String, String> graph2 = new DefaultGraph<String, String>();
        Node<String, String> node6 = graph2.addNode("A");
        Node<String, String> node7 = graph2.addNode("B");
        Node<String, String> node8 = graph2.addNode("C");
        Node<String, String> node9 = graph2.addNode("D");

        graph2.addEdge(node7, node8, "-");
        graph2.addEdge(node8, node9, "-");
        graph2.addEdge(node9, node7, "-");
        graph2.addEdge(node6, node7, "-");
        graph2.addEdge(node6, node8, "-");

        DefaultGraph<String, String> pattern = new DefaultGraph<String, String>();
        Node<String, String> node10 = pattern.addNode("B");
        Node<String, String> node11 = pattern.addNode("C");
        Node<String, String> node12 = pattern.addNode("D");

        pattern.addEdge(node10, node11, "-");
        pattern.addEdge(node11, node12, "-");
        pattern.addEdge(node12, node10, "-");

        FingerprintBuilder fpBuilder = new FingerprintBuilder(2048);
        List<GraphContainer<String, String, DefaultGraph<String, String>>> graphs = new LinkedList<>();
        GraphContainer<String, String, DefaultGraph<String, String>> gc1 = new GraphContainer<>(graph1);
        gc1.setFp(fpBuilder.getFingerprint(graph1));
        graphs.add(gc1);
        GraphContainer<String, String, DefaultGraph<String, String>> gc2 = new GraphContainer<>(graph2);
        gc2.setFp(fpBuilder.getFingerprint(graph2));
        graphs.add(gc2);

        PrefilteredSC<String, String, DefaultGraph<String, String>> sc = new PrefilteredSC<>(fpBuilder,
                new MutableInt());

        Assert.assertEquals("Suport is not equal to 0.5", 0.5, sc.support(graphs, pattern), 0.001);
        Assert.assertTrue("pattern is not supported, while it should be", sc.supported(graphs, pattern, 0.4));
        Assert.assertFalse("pattern is supported, while it should not be", sc.supported(graphs, pattern, 0.6));
    }

    /**
     * Simple test for multiple supported structures. (Test by example)
     */
    @org.junit.Test
    public void MultipeStructureTest() {
        DefaultGraph<String, String> graph1 = new DefaultGraph<String, String>();
        Node<String, String> node1 = graph1.addNode("A");
        Node<String, String> node2 = graph1.addNode("B");
        Node<String, String> node3 = graph1.addNode("C");
        Node<String, String> node4 = graph1.addNode("D");
        Node<String, String> node5 = graph1.addNode("D");

        graph1.addEdge(node1, node2, "-");
        graph1.addEdge(node1, node4, "-");
        graph1.addEdge(node2, node3, "=");
        graph1.addEdge(node2, node4, "-");
        graph1.addEdge(node2, node5, "-");
        graph1.addEdge(node3, node5, "-");
        graph1.addEdge(node4, node5, "-");

        DefaultGraph<String, String> graph2 = new DefaultGraph<String, String>();
        Node<String, String> node6 = graph2.addNode("A");
        Node<String, String> node7 = graph2.addNode("B");
        Node<String, String> node8 = graph2.addNode("C");
        Node<String, String> node9 = graph2.addNode("D");

        graph2.addEdge(node7, node8, "-");
        graph2.addEdge(node8, node9, "-");
        graph2.addEdge(node9, node7, "-");
        graph2.addEdge(node6, node7, "-");
        graph2.addEdge(node6, node8, "-");

        DefaultGraph<String, String> pattern = new DefaultGraph<String, String>();
        Node<String, String> node10 = pattern.addNode("B");
        Node<String, String> node11 = pattern.addNode("C");
        Node<String, String> node12 = pattern.addNode("D");

        pattern.addEdge(node10, node11, "-");
        pattern.addEdge(node11, node12, "-");
        pattern.addEdge(node12, node10, "-");

        FingerprintBuilder fpBuilder = new FingerprintBuilder(2048);
        List<GraphContainer<String, String, DefaultGraph<String, String>>> graphs = new LinkedList<>();
        GraphContainer<String, String, DefaultGraph<String, String>> gc1 = new GraphContainer<>(graph1);
        gc1.setFp(fpBuilder.getFingerprint(graph1));
        graphs.add(gc1);
        graphs.add(gc1);
        graphs.add(gc1);
        graphs.add(gc1);
        graphs.add(gc1);
        graphs.add(gc1);
        graphs.add(gc1);
        GraphContainer<String, String, DefaultGraph<String, String>> gc2 = new GraphContainer<>(graph2);
        gc2.setFp(fpBuilder.getFingerprint(graph2));
        graphs.add(gc2);
        graphs.add(gc2);
        graphs.add(gc2);

        PrefilteredSC<String, String, DefaultGraph<String, String>> sc = new PrefilteredSC<>(fpBuilder,
                new MutableInt());

        Assert.assertEquals("Suport is not equal to 0.5", 0.3, sc.support(graphs, pattern), 0.001);
        Assert.assertTrue("pattern is not supported, while it should be", sc.supported(graphs, pattern, 0.2));
        Assert.assertTrue("pattern is not supported, while it should be", sc.supported(graphs, pattern, 0.3));
        Assert.assertFalse("pattern is supported, while it should not be", sc.supported(graphs, pattern, 0.4));
    }
}
